# Routine prinicpale crant le rapport

html.rapport <- function (brio, demandes, refus, bavard=FALSE, seuil=0.9, file="", date.debut="?", date.fin="?")	{

##################################################
#
# routines utilitaires
#
##################################################


# HTML

# transformation des caractres accentus dans les labels des variables

	html.translate <- function (text)	{
		text <- gsub (pattern="<", replacement="&lt;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern=">", replacement="&gt;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&agrave;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&acirc;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&eacute;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&egrave;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&ecirc;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&icirc;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&ocirc;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&ugrave;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&ucirc;", text, extended=FALSE, perl=FALSE)
		text <- gsub (pattern="", replacement="&ccedil;", text, extended=FALSE, perl=FALSE)
		text
	}

	html.begin <- function (brio, file)	{
		cat ("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\"\n\t\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"fr\" lang=\"fr\">\n<head>\n", file=file) 
		cat ("\t<title>Indicateurs BRIO : ", brio, "</title>\n", sep="", file=file, append=TRUE)
		cat ("\t<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\" />\n", file=file, append=TRUE)
		cat ("<style type=\"text/css\">\ntable {\nborder-style: solid ;\n\tborder-width: 1px ;\n\tbackground-color: #A0A0A0 ;\n}\n\nth {\n\ttext-align: center ;\n\tborder-style: solid ;\n\tborder-width: 1px ;\n\tbackground-color: #FFFFFF ;\n\ttext-align: left;\n}\n\nth.col {\n\ttext-align: center;\n}\n\ntd {\n\tborder-style: solid ;\n\tborder-width: 1px ;\n\ttext-align: right ;\n\tfont-family: courier, monospace ;\n\tpadding-left:10px ;\n\tpadding-right:10px ;\n\tbackground-color: #FFFFFF ;\n}\n\n\n.interet {font-weight: bold;}\n</style>\n", file=file, append=TRUE)
		cat ("</head>\n<body>\n", file=file, append=TRUE)
		cat ("<h1><a name=\"top\">Indicateurs BRIO</a></h1>\n", sep="", file=file, append=TRUE)
		cat ("<p>Liste des indicateurs&nbsp;:</p>", sep="", file=file, append=TRUE)
		cat ("<p>", sep="", file=file, append=TRUE)

		for (i in 1:25)	{
			cat (" <a href=\"#", i, "\">", i, "</a>", sep="", file=file, append=TRUE)
		}
		cat ("</p>", sep="", file=file, append=TRUE)
	}

	html.end <- function (file)	{
		cat ("\n</body>\n</html>\n", file=file, append=TRUE)
	}

	html.p <- function (text, file)	{
		cat ("\n<p>", text, "</p>\n", sep="", file=file, append=TRUE)
	}

	html.ul <- function (stringList)	{
		if (!is.null(stringList))	{
			ul <-	paste (
				"<ul>\n",
				paste ("<li>", stringList, "</li>\n", sep="", collapse=""),
				"</ul>\n",
				sep=""
			)
		} else	{
			ul <- ""
		}
		ul
	}
	html.ul (c("a", "b", "c"))
	html.ol <- function (itemList)	{
		ol <- ""
		itemList[itemList!=""] <- paste (
			"<li>",
			itemList[itemList!=""],
			"</li>\n",
			sep=""
		)
		itemList <- paste (itemList, collapse="")
		if (itemList != "")	{
			ol <- paste (
				"<ol>\n",
				itemList,
				"</ol>\n",
				sep=""
			)
		}
		ol
	}


# Affichage du contenu d'une matrice sous forme de tableau HTML


	html.table <- function (x, titre.lignes=NULL, ytitle=NULL, interet=NULL)	{	
		make.row<- function (x.i)	{
			row.text <- paste (c("<tr>\n\t", paste (x.i, sep=""), "\n</tr>\n"), sep="")
			row.text
		}

		x <- as.matrix (x)
		m <- nrow(x)
		n <- ncol(x)
	
		x.labels <- NULL
		y.labels <- NULL
		if (!is.null(dimnames(x)))	{
			x.labels <- dimnames(x)[[1]]
			y.labels <- dimnames(x)[[2]]
		}
		
		m.0 <- as.integer (!is.null(ytitle)) + as.integer(!is.null (y.labels))
		n.0 <- as.integer (!is.null(titre.lignes)) + as.integer(!is.null (x.labels))
		if (!is.null(y.labels))	{
			y.labels <- paste ("<th>", y.labels, "</th>", sep="")
		}
		if (!is.null(x.labels))	{
			x.labels <- paste ("<th>", x.labels, "</th>", sep="")
		} else	{
			x.labels <- rep ("", m)
		}
		x <- matrix (paste ("<td>",  x, "</td>", sep=""), nrow=m)
		text <- ""
	
		# taille de la cellule suprieure gauche (vide)
		cellule.sup.gauch <- ""
		if (m.0>0 && n.0>0) cellule.sup.gauch <- paste ("<th colspan=\"", n.0, "\" rowspan=\"", m.0, "\"></th>", sep="")
		
		# titre gnral pour les colonnes (facultatif)
		if (!is.null(ytitle))	{
			titre.colonnes <- make.row (
				paste(
					cellule.sup.gauch,
					"<th colspan=\"",
					n,
					"\">",
					ytitle,
					"</th>",
					collapse=""
				)
			)
		} else	{
			titre.colonnes <- ""
		}

		# intituls pour chaque colonne (facultatif)
		if (!is.null(y.labels))	{
			intitule.colonnes <- make.row (
				c(
					ifelse(is.null(ytitle),cellule.sup.gauch,""),
					y.labels
				)
			)
		}
		# titre gnral pour les lignes (facultatif)		
		if (!is.null(titre.lignes))	{
			titre.lignes <- paste ("<th rowspan=\"", m, "\">", titre.lignes, "</th>", sep="")
		} else	{
			titre.lignes <- ""
		}
		
		# premire ligne, avec titre gnral si ncessaire
		ligne.1 <- make.row(
			paste(c(titre.lignes, x.labels[1],x[1,]), sep="")
		)
	
		# lignes suivantes, avec intitul facultatif
		lignes.n <- paste (
			apply (cbind(x.labels[-1],x[-1,,drop=FALSE]), MARGIN=1, FUN=make.row),
			sep=""
		)
		text <- paste (
			"<table>\n",
			paste (titre.colonnes, collapse=""),
			paste (intitule.colonnes, collapse=""),
			paste (ligne.1, collapse=""),
			paste (lignes.n, collapse=""),
			"</table>\n",
			sep=""
		)
		text
	}




# Analyses

# taux de rponse

	taux.reponse <- function (x, souspop=NULL, taille.souspop=NULL, seuil)	{

		# prparation des variables
		if (!is.matrix(x))	{
			if (is.list(x))	{
				x <- matrix(
					unlist(x),
					ncol=length(x),
					dimnames=list(NULL,sapply(x,FUN=attr,"description",simplify=TRUE))
				)
			} else	{
				x <- matrix (
					x,
					ncol=1,
				dimnames=list(NULL,attr(x,"description"))
				)
			}
		}
		if (!is.null(souspop))	{
			x <- x[souspop,,drop=FALSE]
		}
		if (is.null(taille.souspop))	{
			taille.souspop <- NROW(x)
		}
		# calcul des taux de rponse
		# Calcule le taux de rponse individuel d'une seule variable ou conjoint si plusieurs sont fournies.
		# x : matrice contenant les variables en vecteurs-colonne.
		# Si la taille de la sous-population n'est pas spcifie, le nombre de lignes de x fait foi.
		taux <- function (x, taille.souspop=NULL)	{
			if (is.null(taille.souspop))	{
				n <- NROW(x)
			} else	{
				n <- taille.souspop
			}
			if (n > 0)	{
				r <- sum(rowSums(is.na(as.matrix(x)))==0)
				rn <- signif (as.double(r) / as.double(n), digits=2)
			} else	{
				# collectif vide
				r <- 0
				rn <- NA
			}
			tr <- matrix (
				c(rn,r,n),
				nrow=3,
				dimnames=list(c("rn","r","n"), NULL)
			)
			tr
		}
		taux.individuels <- matrix (
			apply (X=x, MARGIN=2, FUN=taux, taille.souspop=taille.souspop),
			ncol=NCOL(x),
			dimnames=list(c("rn","r","n"), dimnames(x)[[2]])
		)
		taux.conjoint <- taux (x=x, taille.souspop=taille.souspop)

		# liste des variables ne remplissant pas les critres d'exhaustivit
		var.manquantes <- NULL
		if (is.na(taux.conjoint[1,1]))	{
			var.manquantes <- dimnames(taux.individuels)[[2]][!is.na(taux.individuels[1,])&taux.individuels[1,]<1.0]
		} else	{
			var.manquantes <- dimnames(taux.individuels)[[2]][taux.individuels[1,]<seuil]
		}
		if (length(var.manquantes) == 0)	{
			var.manquantes <- NULL
		}
		tr <- list (
			conjoint=taux.conjoint,
			individuel=taux.individuels,
			var.manquantes=var.manquantes,
			seuil=seuil
		)
		class(tr) <- "taux.reponse"
		tr
	}


	# Formatte le taux de rponse en chane de caractres HTML.
	# Arguments :
	# tr 		: matrice 3x1 telle que calcule par taux.reponse (r/n, r, n).
	# seuil	: taux de reponse minimal requis
	# Valeur :
	# chane de caractres

	html.taux.reponse <- function (tr, seuil)	{
		tr <- as.vector(tr)
		rn <- tr[1]
		r <- tr[2]
		n <- tr[3]
		if (!is.na(rn))	{
			# collectif non vide, le taux de rponse a un sens
			html.tr <-  100.0*rn

			if (rn < seuil)	{
				# mise en vidence des taux insuffisants
				html.tr <- paste (c("<strong>",html.tr, "</strong>"), sep="")
			}
			html.tr <- paste (c(html.tr, "%"), collapse="")
		} else	{
			html.tr <- paste ("collectif vide")
		}
		html.tr <- paste (html.tr, " (", r, "/", n, ")", sep="")
		html.tr
	}



	detail.collectif <- function (
		description="Demandes d orientation m&eacute;dico-sociale.",
		variables.souspop=NULL,
		modalites.souspop=NULL,
		taille.souspop=NULL,
		seuil
	)	{
		# initialisation de l objet collectif rendu, avec des valeurs qui n engagent  rien
		collectif <- list (
			taux.reponse=NULL,
			description=description,
			variables=NULL,
			modalites=NULL,
			indicateur=NULL
		)
		if ((!is.null(variables.souspop) && !is.null(modalites.souspop)) || !is.null(taille.souspop))	{

			if (!is.null(variables.souspop) && !is.null(modalites.souspop))	{
				# prparation des donnes : tout doit tre sous forme de liste
				if (!is.list(variables.souspop)) variables.souspop <- list(variables.souspop)
				if (!is.list(modalites.souspop)) modalites.souspop <- list(modalites.souspop)
	
				collectif$taux.reponse <- taux.reponse (x=variables.souspop, seuil=seuil)
	
				collectif$indicateur=rep(TRUE, length(variables.souspop[[1]]))
				collectif$variables <- list()
				collectif$modalites <- list()
				for (i in 1:length(variables.souspop))	{
					# pour chaque variable, reprer les observations faisant partie du collectif
					collectif$indicateur <- collectif$indicateur & !is.na(match(as.integer(variables.souspop[[i]]), modalites.souspop[[i]]))
					# description des variables et des modalits servant  dfinir le collectif
					collectif$variables[[i]] <- attr(variables.souspop[[i]], "description")
					collectif$modalites[[i]] <- levels(variables.souspop[[i]] )[modalites.souspop[[i]]]
				}
			} else	{
				collectif$indicateur <- rep (TRUE, taille.souspop)
				collectif$taux.reponse <- taux.reponse (x=collectif$indicateur, seuil=seuil)
			}
		}
		class(collectif) <- "collectif"
		collectif
	}

	html.detail.collectif <- function (collectif, bavard=TRUE)	{
		if (!is.null(collectif$description))	{
			description.html <- paste (
				"<strong>Collectif :</strong> ",
				collectif$description,
	 			sep=""
			)
		} else	{
			description.html <- NULL
		}
		if (!is.null(collectif$taux.reponse) && (bavard == TRUE || collectif$taux.reponse$conjoint[1,1] < collectif$taux.reponse$seuil))	{
			tr <- apply (X=collectif$taux.reponse$individuel, MARGIN=2, FUN=html.taux.reponse, seuil=collectif$taux.reponse$seuil)
			variables <- unlist (collectif$variables)
			modalites <- lapply(X=collectif$modalites, FUN=html.ul)
			details <- paste (
				variables,
				" [", tr, "]",
				modalites,
				sep=""
			)
			details <- paste (
				"\n<br />Variables, taux de r&eacute;ponse individuels ",
				ifelse (bavard==TRUE, "et modalit&eacute;s ", ""),
				"servant &agrave; d&eacute;finir le collectif :\n",
				html.ul (details)
			)
			if (NCOL(collectif$taux.reponse$individuel) > 1)	{
				details <- paste (
					details,
					"Taux de r&eacute;ponse conjoint : ",
					html.taux.reponse (collectif$taux.reponse$conjoint, seuil=collectif$taux.reponse$seuil),
					sep=""
				)
			description.html <- paste (description.html, details, sep="")
			}
		}
		description.html
	}

	detail.indicateurs <- function (variables, indicateur.souspop=NULL, seuil)	{
		indicateurs <- list (taux.reponse=NULL, variables =NULL ,var.manquantes=NULL)
		class(indicateurs) <- "indicateurs"	

		if (!is.null(variables))	{
			if (!is.list(variables))	{
				variables <- list(variables)
			}
			if (is.null(indicateur.souspop))	{
				indicateur.souspop <- rep (TRUE, length(variables[[1]]))
			}
			indicateurs$taux.reponse <- taux.reponse (x=variables, souspop=indicateur.souspop, seuil=seuil)
			indicateurs$variables <- sapply (variables, FUN=attr, which="description", simplify=TRUE)
		}
		indicateurs
	}

	html.detail.indicateurs <- function (indicateurs, bavard=TRUE)	{
		if (!is.null(indicateurs$taux.reponse) && (bavard==TRUE || (!is.na(indicateurs$taux.reponse$conjoint) && indicateurs$taux.reponse$conjoint < indicateurs$taux.reponse$seuil)))	{
			description.indicateurs <- "<strong>Variable(s)</strong> servant au calcul de l'indicateur et taux de r&eacute;ponse calcul&eacute; sur le collectif :\n"
			description.indicateurs <- paste(
				description.indicateurs,
				html.ul (
					paste (
						indicateurs$variables,
						" [",
						apply (X=indicateurs$taux.reponse$individuel, MARGIN=2, FUN=html.taux.reponse, seuil=indicateurs$taux.reponse$seuil),
						"]",
						sep=""
					)
				)
			)
			if (NCOL(indicateurs$taux.reponse$individuel) > 1)	{
				description.indicateurs <- paste (
					description.indicateurs,
					"Taux de r&eacute;ponse conjoint : ",
					html.taux.reponse (indicateurs$taux.reponse$conjoint, indicateurs$taux.reponse$seuil),
					sep=""
				)			
			}
		} else	{
			description.indicateurs <- NULL
		}
		description.indicateurs
	}



	html.indicateur <- function (
		no=NULL,
		descr.indicateur=NULL,
		souspop=NULL,
		souspop.description=NULL,
		variables.souspop=NULL,
		modalites.souspop=NULL,
		variables=NULL,
		tableau.special=NULL,
		file="",
		lignes.ordonnees=FALSE,
		interet=NULL,
		lignes.exclues=NULL,
		pas.cellules.vides=FALSE,
		cumul=FALSE,
		seuil=0.0,
		inside.supplement=NULL,
		outside.supplement=NULL,
		showStandardTable=TRUE,
		bavard.taux=TRUE,
		bavard.tableau=TRUE

	)	{
		html.top <- function(file="")	{
			cat ("<p><a href=\"#top\">&uarr;</a></p>\n", file=file, append=TRUE)
		}
		html.h3 <- function (id, file)	{
			cat ("\n<hr /><h3><a name=\"", id,"\">Indicateur ", id, "</a></h3>\n", sep="", file=file, append=TRUE)
		}

		if (!is.null(variables) && !is.list(variables)) variables <- list(variables)
	
		# titre et description
	
		if (!is.null(no))	{
			html.h3(no, file=file)
		}
		if (!is.null(descr.indicateur))	{
			html.p (descr.indicateur, file=file)
		}
	
		# collectif
	
		if (is.null(souspop))	{
			if (!is.null(variables))	{
				taille.souspop <- length(variables[[1]])
			} else	{
				taille.souspop <- NULL
			}
			souspop <- detail.collectif (
				description=souspop.description,
				variables.souspop=variables.souspop,
				modalites.souspop=modalites.souspop,
				taille.souspop=taille.souspop ,
				seuil=seuil
			)
		}
		# criteres
	
		indicateurs <- detail.indicateurs (variables=variables, indicateur.souspop=souspop$indicateur, seuil=seuil)

		# Dresser la liste des varaiables qui posent problme, tant dans la dfinition du collectif que de l'indicateur lui-mme.
		var.manquantes <- c (
			souspop$taux.reponse$var.manquantes,
			indicateurs$taux.reponse$var.manquantes
		)

		cat (
			html.ul(c(
				html.detail.collectif(souspop, bavard=bavard.taux),
				html.detail.indicateurs(indicateurs, bavard=bavard.taux)
			)),
			append=TRUE,
			file=file
		)
		# si les taux de rponse sont suffisants, calculer l'indicateur
		if (
			(is.null(souspop$taux.reponse) || (!is.na(souspop$taux.reponse$conjoint[1,1]) && souspop$taux.reponse$conjoint[1,1]>=seuil))
			&& (is.null(indicateurs$taux.reponse) || (!is.na(indicateurs$taux.reponse$conjoint[1,1]) && indicateurs$taux.reponse$conjoint[1,1]>=seuil))
		)	{
			calcul.possible <- TRUE
			ylabel <- NULL
			xlabel <- NULL
			tableau <- NULL
			if (!is.null(variables) && showStandardTable==TRUE)	{
				x <- variables[[1]]
				# une ou deux variables d'intrt (tableau crois) ?
				if (length(variables) > 1)	{
					y <- variables[[2]]
					xlabel <- attr(x,"description")
					ylabel <- attr(y,"description")
					tableau.standard <- tableau.bivarie (
						x=x[souspop$indicateur],
						y=y[souspop$indicateur],
						pas.cellules.vides=pas.cellules.vides,
						lignes.exclues=lignes.exclues,
						cumul=cumul
					)
				} else	{
					tableau.standard <- tableau.univarie (
						x=x[souspop$indicateur],
						titre.lignes=attr(x,"description"),
						pas.cellules.vides=pas.cellules.vides,
						interet=interet,
						lignes.ordonnees=lignes.ordonnees,
						lignes.exclues=lignes.exclues,
						cumul=cumul,
						bavard=bavard.tableau
					)
				}
				# univari : html.table (t$table, file=file)
				if (bavard.tableau == TRUE)	{
					if (!is.null(tableau.standard$somme.interet))	{
						html.p (paste("Proportion cumul&eacute;e des modalit&eacute;s d'int&eacute;r&ecirc;t : <strong>", tableau.standard$somme.interet, "%</strong>", sep=""), file=file)
					}
					tableau <- tableau.standard$table
				} else	{
					if (!is.null(tableau.standard$somme.interet))	{
						html.p (paste("<table>\n<tr><td>", tableau.standard$somme.interet, "</td></tr>\n</table>\n", sep=""), file=file)
					} else	{
						tableau <- tableau.standard$table
					}
				}
			}
			if (!is.null(tableau.special))	{
				tableau <- tableau.special
			}
			if (!is.null(tableau))	{
				cat (html.table (tableau, titre.lignes=xlabel, ytitle=ylabel), file=file, append=TRUE)
			}
			if (!is.null(inside.supplement))	{
				cat (inside.supplement, sep="", append=TRUE, file=file)
			}
		} else	{
			calcul.possible<- FALSE
			cat ("<table>\n<tr>\n\t<th>Collectif vide ou taux de r&eacute;ponse insuffisant pour calculer l'indicateur.</th>\n</tr>\n</table>\n", file=file, append=TRUE)

			# Sauver les cas pour lesquels le collectif est vide parce qu'il n'y a pas d'observations
			# rpondant aux critres (mais les taux de rponse nuls ne sont pas accepts !).

			if (
				is.null(indicateurs$taux.reponse)
				|| (indicateurs$taux.reponse$conjoint[3,1]==0 && souspop$taux.reponse$conjoint[2,1]>0)
			)	{
				# ne pas pnaliser pour des collectifs vides
				calcul.possible <- TRUE
			}
		}
		if (!is.null(outside.supplement))	{
			cat (outside.supplement, "\n", sep="", file=file, append=TRUE)
		}
		html.top (file)

		list (var.manquantes=var.manquantes, calcul.possible=calcul.possible)
	}


	tableau.univarie <- function (x, titre.lignes="", pas.cellules.vides=FALSE, interet=NULL, lignes.ordonnees=FALSE,  lignes.exclues=NULL, cumul=FALSE, bavard=TRUE)	{
		
		# cell count
		n <- as.matrix(table(x))
		l <- matrix (dimnames(n)[[1]], ncol=1)
		# layout
		if (is.null(interet))	{
			interet <- rep (FALSE, nrow(n))
		} else	{
			if (class(interet) != "logical")	{
				if (interet[1] == 0)	{
					interet <- rep(TRUE,nrow(n))
				} else	{
					interet <- !is.na (match(1:nrow(n), interet))
				}
			}
		}
		lignes.a.garder <- rep (TRUE, nrow(n))
		if (!is.null (lignes.exclues))	{
			if (class(lignes.exclues) != "logical")	{
				lignes.exclues <- !is.na (match(1:nrow(n), lignes.exclues))
			}
			lignes.a.garder <- !lignes.exclues
		}
		if (pas.cellules.vides == TRUE)	{
			lignes.a.garder[n[,1]==0] <- FALSE
		}
		if (lignes.ordonnees == TRUE)	{
			reorder <- order(as.integer(n[,1]), decreasing=TRUE)
			n <- n[reorder,,drop=FALSE]
			l <- l[reorder,,drop=FALSE]
			interet <- interet[reorder]
			lignes.a.garder <- lignes.a.garder[reorder]
		}
		n <- n[lignes.a.garder,,drop=FALSE]
		l <- l[lignes.a.garder,,drop=FALSE]
		interet <- interet[lignes.a.garder]
		s <- sum(n)
		n <- rbind(n, s)
		l <- rbind(l,"TOTAL")
		dimnames(n) <- list (NULL, c("Fr&eacute;q."))
		# relative frequency
		f <-  n / s
		if (cumul== TRUE)	{
			f[-nrow(f),1] <- cumsum (f[-nrow(f),1])
			dimnames(f)[[2]] <- c("Pourc. cum.")
		} else	{
			dimnames(f)[[2]] <- c("Pourc.")

		}
		if (length(interet) < nrow(f))	{
			interet <- c(interet, FALSE)
		}
		if (bavard == TRUE)	{
			if (sum(as.integer(interet)) > 1 && sum(as.integer(interet)) < nrow(f)-1)	{
				somme.interet <- sum(f[interet,1])
				somme.interet <- format (100.0*somme.interet, nsmall=1 ,digits=1)
			} else	{
				somme.interet <- NULL
			}
		} else	{
			if (sum(as.integer(interet)) >= 1 && sum(as.integer(interet)) < nrow(f)-1)	{
				somme.interet <- paste (
					"<strong>",
					format (100.0*sum(f[interet,1]), nsmall=1 ,digits=1),
					" %</strong> (",
					sum(n[interet,1]),
					"/",
					s,
					")",
					sep=""
				)
			} else	{
				somme.interet <- NULL
			}
		}
		f <- format (100.0*f, nsmall=1 ,digits=1)
		f[interet] <- paste ("<span class=\"interet\">", f[interet], "</span>", sep="")
		dimnames(l) <- list (NULL, titre.lignes)
		l[interet] <- paste ("<span class=\"interet\">", l[interet], "</span>", sep="")

		t <- list (table=cbind(l, n, f), somme.interet=somme.interet)
		t
	}

	tableau.bivarie <- function (x, y, pas.cellules.vides=FALSE, lignes.exclues=NULL, cumul=FALSE)	{
		
		n <- as.matrix (table(x,y))
		if (cumul==FALSE)	{
			s <- sum(n)
		}
		n <- cbind (n, apply (n, MARGIN=1, FUN=sum))
		n <- rbind (n, apply (n, MARGIN=2, FUN=sum))
		if (cumul == TRUE)	{
			f <- n / as.vector(n[,NCOL(n)])
		} else 	{
			f <- n / s
		}
		f <- paste ("<span class=\"interet\">", format(100.0*f,nsmall=1,digits=1), "</span>", sep="")
		t <- matrix (paste (as.vector(n), f, sep="<br />"), ncol=nlevels(y)+1)
		
		if (pas.cellules.vides==TRUE)	{
			n <- n[n[,ncol(n)]>0,n[nrow(n),]>0]
		}
		
		dimnames(t) <- dimnames(n)
		dimnames(t)[[1]][nrow(t)] <- dimnames(t)[[2]][ncol(t)] <- "TOTAL"
		t <- list (table=t, somme.interet=NULL)
	}


tabulate <- function (x, y=NULL, file="", label=NULL,  ylabel=NULL, pas.cellules.vides=FALSE, interet=NULL, lignes.ordonnees=FALSE, lignes.exclues=NULL, cumul=FALSE)	{
	# univari
	if (is.null(y))	{
		t <- tableau.univarie (x, titre.lignes=label, pas.cellules.vides=pas.cellules.vides, interet=interet, lignes.ordonnees=lignes.ordonnees, lignes.exclues=lignes.exclues, cumul=cumul)
#		html.table (t$table, file=file)
#		if (!is.null(t$somme.interet))	{
#			html.p (paste("Proportion cumul&eacute;e des modalit&eacute;s d'int&eacute;r&ecirc;t : <strong>", t$somme.interet, "%</strong>", sep=""), file=file)
#		}
		# bivari
	} else	{
		t <- tableau.bivarie (x, y, cumul=FALSE)
#		html.table (t, file=file, titre.lignes=label, ytitle=ylabel)
	}
	t
}

###################################################
#
# variables ncessaires au calcul des indicateurs
#
###################################################

	# codage dichotomique des AVQ

	sommeavq <- as.integer(demandes$manger) + as.integer(demandes$laver) + as.integer(demandes$habiller) + as.integer(demandes$deshab) + as.integer(demandes$entpers) + as.integer(demandes$contves) + as.integer(demandes$contana) + as.integer(demandes$toilet)
	degreavq <- factor (1+as.integer(sommeavq>8), levels=c(1,2), labels=c("Autonome", "D&eacute;pendant"), ordered=TRUE)
	attr(degreavq, "description") <- "Degr&eacute; d'AVQ"

	# codage du degr de cognition
	
	degrecog <- rep (x=NA, times=nrow(demandes))
	temp <- apply (cbind(as.integer(demandes$memoire), as.integer(demandes$orientat), as.integer(demandes$jugement), as.integer(demandes$soccuper)), MARGIN=1, FUN=max)
	degrecog[temp==1] <- 1
	degrecog[temp==2] <- 2
	degrecog[temp>=3] <- 3
	rm (temp)
	degrecog <- factor (degrecog, levels=1:3, labels=c("Autonome", "Surveillance/aide", "D&eacute;pendant"), ordered=TRUE)
	attr(degrecog, "description") <- "Etat mental"

	# variable de transfert
  # Si j'ai bien compris (OB), on crit d'abord 2 pour "Demande d'orientation mdico-sociale"  toutes les lignes,
  # puis on remplace par les autres valeurs s'il y a lieu
  transfer <- rep (x=2, times=nrow(demandes))
	transfer[(as.integer(demandes$provient)==1&as.integer(demandes$aboutiss)==6)|(as.integer(demandes$provient)==2&as.integer(demandes$aboutiss)==6)|(as.integer(demandes$provient)==1&as.integer(demandes$aboutiss)==7)|(as.integer(demandes$provient)==2&as.integer(demandes$aboutiss)==7)] <- 1
	transfer[is.na(demandes$aboutiss) | as.integer(demandes$aboutiss)==11] <- 3
	transfer[as.integer(demandes$aboutiss)==8|as.integer(demandes$aboutiss)==9|as.integer(demandes$aboutiss)==10] <- 4
	transfer[is.na(demandes$provient)] <- NA
	transfer[!is.na(demandes$dateabou) & is.na(demandes$aboutiss)] <- NA
	transfer <- factor (transfer, levels=c(1,2,3,4), labels=c("Transferts A-B", "Demandes d'orientation m&eacute;dico-sociale", "Demandes en cours", "Non abouties (abandon, d&eacute;c&egrave;s, formulation d'une nouvelle demande)"))
	attr(transfer, "description") <- "Demandes"
	# region du rseau
	# attention : toutes les bornes sont dcales de +1 par rapport aux niveaux originaux de la variable "rgion",
	# suite aux consquences des la catgorisation.

	reseau <- rep (NA, 54)
	reseau[1] <- "HORS RESEAU"
	reseau[c(2:11,13:14,19:21)] <- "ARCOS"
	reseau[c(12,15:18,27:33)] <- "ARC"
	reseau[c(22:26,54)] <- "ASCOR"
	reseau[34:39] <- "AROVAL"
	reseau[40:45] <- "RENOVA"
	reseau[46:50] <- "SANTE-BROYE"
	reseau[51:53] <- "CHABLAIS"
  #reseau[c(34:39,40:45,46:50)] <- "RNB"  
	reseau <- factor (reseau)
	regres <- 2 - as.integer(reseau[as.integer(demandes$region)]==brio)
	regres <- factor (regres, levels=1:2, labels=c("R&eacute;seau", "Hors r&eacute;seau"))
	attr (regres, "description") <- "R&eacute;gion du r&eacute;seau"

# population dans les rgions des rseaux (donnes fournies par le SCRIS)

	pop.res <- matrix (
		c(
			238654, 21404, 14832, 5865,
			111278, 7929, 4996, 1881,
			59055, 6307, 4612, 2049,
			39724, 3474, 2418, 880,
			39491, 4053, 3007, 1074,
			27311, 2625, 1872, 618,
			29599, 2966, 2001, 682
	    #106526, 10152, 7297, 2572 
    ),
		#nrow=8,
		nrow=7,
    ncol=4,
		byrow=TRUE,
		dimnames=list(
			#c("ARCOS", "ARC", "ASCOR", "AROVAL", "RENOVA", "SANTE-BROYE", "CHABLAIS", "RNB"),
			c("ARCOS", "ARC", "ASCOR", "AROVAL", "RENOVA", "SANTE-BROYE", "CHABLAIS"),
      c("&lt;65", "65-74", "75-84", "&gt;=85")
		)
	)

##############################################################
#
# affichage des rsultats
#
##############################################################

# Dbut

	html.begin (brio=brio, file=file)

	html.p (paste("BRIO :", brio, collapse=" "), file=file)
	html.p (paste ("P&eacute;riode du ", date.debut, " au ", date.fin, ".", sep=""), file=file)
	html.p (paste ("Nombre de demandes enregistr&eacute;es pour la p&eacute;riode : ",nrow(demandes), ".", sep=""), file=file)
	html.p (paste ("Nombre de demandes non classifiables : ", sum(is.na(transfer)), " (", format(100.0*sum(is.na(transfer)) / length(transfer),nsmall=1,digits=1), "%).", sep=""), file=file)
	html.p (paste ("Taux de r&eacute;ponse minimal pour le calcul des indicateurs :", format(100.0*seuil ,nsmall=1,digits=1), "%.", sep=" "), file=file)

	html.p ("Distribution du nombre de demandes par usager :", file=file)
	dusager <- table(table(demandes$idusager))
	dusager <- cbind (names(dusager), as.character(dusager), format(100.0*dusager/sum(dusager),nsmall=1,digits=1))
	dimnames (dusager) <- list (NULL, c("Demandes","Usagers","Pourc."))
	cat(html.table (dusager), file=file, append=TRUE)
	rm (dusager)

	if (bavard == TRUE) {
		html.p ("D&eacute;tail des usagers ayant formul&eacute; plusieurs demandes :", file=file)
		details <- demandes[!is.na(match (table=demandes$idusager[duplicated(demandes$idusager)], x=demandes$idusager)), c(1,34,38,40,45,48,47)]
		details <- as.matrix(details)
		details <- details [order(details[,1,drop=FALSE], details[,2,drop=FALSE]),]
		details[c(FALSE, details[-nrow(details),1]==details[-1,1]),1] <- ""
		details[is.na(details)] <- "-"
		dimnames(details) <- list( NULL, c(
			attr(demandes[[1]], "description"),
			attr(demandes[[34]], "description"),
			attr(demandes[[38]], "description"),
			attr(demandes[[40]], "description"),
			attr(demandes[[45]], "description"),
			attr(demandes[[48]], "description"),
			attr(demandes[[47]], "description")
		))
		
		cat(html.table (details), file=file, append=TRUE)
		rm (details)
	}

	collectif.horsTransfert <- detail.collectif (
		description="Demandes d'orientation m&eacute;dico-sociale.",
		variables.souspop=transfer,
		modalites.souspop=2,
		seuil=seuil
	)
	html.p ("Distribution des demandes :", file=file)
	indic <- html.indicateur (
		variables=transfer,
		file=file,
		seuil=seuil,
		bavard.taux=FALSE,
		bavard.tableau=TRUE
	)
	html.p ("D&eacute;tail des demandes d'orientation m&eacute;dico-sociale :", file=file)
	cat (
		html.table (tabulate (x=demandes$pecdem[collectif.horsTransfert$indicateur], label=attr(demandes$pecdem, "description"))$table),
		file=file,
		append=TRUE
	)

	reponse.faible <- NULL
	indList <- NULL
# 1
	indic <- html.indicateur (
		no=1,
		descr.indicateur="Occupation permanente du poste de responsable du BRIO et de celui de son suppl&eacute;ant.",
		file=file
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
	
# 2

	indic <- html.indicateur (
		no=2,
		descr.indicateur="Proportion des demandes concernant des patients hospitalis&eacute;s en lit A ou B et dont l'aboutissement est un s&eacute;jour en lit A ou B (transferts A-B).",
	 	souspop.description="toutes les demandes enregistr&eacute;es.",
		variables=transfer,
		interet=1,
		file=file,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

# 3 
	collectif.regionReseau <- detail.collectif(
		description="Demandes d'orientation m&eacute;dico-sociale provenant du r&eacute;seau.",
		variables.souspop=list(transfer,regres),
		modalites.souspop=list(2,1),
		seuil=seuil
	)


	tableau.special <- as.matrix(table (demandes$age[collectif.regionReseau$indicateur]))
	tableau.special <- cbind (
		format(tableau.special * (1000.0/pop.res[dimnames(pop.res)[[1]]==brio,]),nsmall=1,digits=1),
		format(tableau.special, nsmall=0, digits=1)
	)
	tableau.special.labels <- as.matrix(paste ("<span class=\"interet\">", dimnames(tableau.special)[[1]], "</span>", sep=""))
	tableau.special <- matrix(
		paste (
			"<span class=\"interet\">",
			tableau.special,
			"</span>",
			sep=""
		),
		ncol=2
	)
	tableau.special <- cbind (tableau.special.labels, tableau.special)
	dimnames (tableau.special) <- list (NULL, c(attr(demandes$age,"description"),"Taux [/1000 hab.]", "n"))

	indic <- html.indicateur (
		no=3,
		descr.indicateur="Taux de demandes pour 1000 habitants, par cat&eacute;gorie d'&acirc;ge.",
		souspop=collectif.regionReseau,
		variables=demandes$age,
		tableau.special=tableau.special,
		file=file,
		seuil=seuil,
		showStandardTable=FALSE,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	rm (tableau.special, tableau.special.labels)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
	
# 4

	ct <- table (demandes$origine[collectif.horsTransfert$ind])

	indic <- html.indicateur (
		no=4,
		descr.indicateur="Distribution de l'origine des demandes.",
		souspop=collectif.horsTransfert,
		variables=demandes$origine,
		file=file,
		interet=1:15,
		inside.supplement=paste("<p>Proportion de demandes dont l'origine est ext&eacute;rieure &agrave; l'h&ocirc;pital A :<strong>", format(100.0*(sum(ct[c(-7,-13)])/sum(ct)),nsmall=1,digits=1), "%</strong> et aux h&ocirc;pitaux A et B :<strong>", format(100.0*(sum(sum(ct[c(-6,-7,-12,-13)])/sum(ct))),nsmall=1,digits=1), "%</strong>.</p>"),
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	rm (ct)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)


# 5
	indic <- html.indicateur(
		no=5,
		descr.indicateur="Distribution de l'&acirc;ge des usagers.",
		souspop=collectif.horsTransfert,
		variables=demandes$age,
		file=file,
		interet=1:4,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	indic <- html.indicateur(
		descr.indicateur="Distribution du sexe des usagers.",
		souspop=collectif.horsTransfert,
		variables=demandes$sexe,
		file=file,
		interet=1:2,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	indic <- html.indicateur(
		descr.indicateur="Distribution de la situation sociale des usagers.",
		souspop=collectif.horsTransfert,
		variables=demandes$sitsoc,
		file=file,
		interet=1:5,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)


	HTMLRegions <- function (x)	{
		rt <-table (x)
		rt <- rt[rt>0,drop=FALSE]

		reg <- matrix(names(rt),ncol=1)
		n <- matrix(rt)
		i <- order(n, decreasing=TRUE)
		reg <- reg[i,,drop=FALSE]
		n <- n[i,,drop=FALSE]
		f <- format(100.0*n/sum(n),nsmall=1,digits=1)

		zone <- factor(as.character(reseau[match (x=reg, table=levels(demandes$region))]))


		ct <- cbind (reg, as.character(n), f)
		ct <- matrix(paste ("<td>", ct , "</td>", sep=""), ncol=ncol(ct))
		ct <- apply (X=ct, MARGIN=1, FUN=paste, collapse="")


		MakeRow <- function (x)	{
			x <- paste ("<tr>\n\t", x, "\n</tr>\n", sep="")
			x <- paste (x, collapse="")
			x
		}
		n.zone <- unlist(tapply (n, INDEX=zone, FUN=sum))
		ct <- unlist(tapply (ct, INDEX=zone, FUN=MakeRow))
		ct <- paste (
			paste (
				"<tr><td><strong>",
				levels(zone),
				"</strong></td><td><strong>",
				n.zone,
				"</strong></td><td><strong>",
				format(100.0*n.zone/sum(n.zone),nsmall=1,digits=1),
				"</strong></td></tr>\n",
				sep=""
			),
			ct,
			sep=""
		)
		ct <- paste (
			"<table>\n<tr><th>R&eacute;gion</th><th>Fr&eacute;q.</th><th>Pourc.</th></tr>\n",
			paste(ct, collapse=""),
			"<tr><td><strong>TOTAL</strong></td><td><strong>",
			sum(n),			
			"</strong></td><td><strong>100.0</strong></td></tr>\n</table>\n",
			sep=""
		)
		ct
	}
	indic <- html.indicateur(
		descr.indicateur="Distribution de la r&eacute;gion de domicile des usagers.",
		souspop=collectif.horsTransfert,
		variables=demandes$region,
		file=file,
		interet=0,
		pas.cellules.vides=TRUE,
		lignes.ordonnees=TRUE,
		seuil=seuil,
		inside.supplement=HTMLRegions (demandes$region[collectif.horsTransfert$ind]),
		showStandardTable=FALSE,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	rm (HTMLRegions)

	indic <- html.indicateur(
		descr.indicateur="Distribution de la provenance des usagers.",
		souspop=collectif.horsTransfert,
		variables=demandes$provient,
		file=file,
		interet=1:10,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)


# 6
	indic <- html.indicateur (
		no=6,
		descr.indicateur="Proportion des demandes concernant des usagers d&eacute;j&agrave; suivis par un CMS ou un autre prestataire de maintien &agrave; domicile.",
		souspop=collectif.horsTransfert,
		variables=demandes$cms,
		file=file,
		interet=1:2,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
	
# 7

	temp <- cbind (demandes$probleme1, demandes$probleme2, demandes$probleme3, demandes$probleme4, demandes$probleme5, demandes$probleme6, demandes$probleme7, demandes$probleme8, demandes$probleme9, demandes$probleme10, demandes$probleme11, demandes$probleme12)
	temp <- temp[collectif.horsTransfert$ind,]
	n <- apply (temp==2, MARGIN=2, FUN=sum, na.rm=TRUE)
	f <- n / apply (!is.na(temp), MARGIN=2, FUN=sum)
	
	prob <- matrix (c(
			attr(demandes$probleme1,"description"),
			attr(demandes$probleme2,"description"),
			attr(demandes$probleme3,"description"),
			attr(demandes$probleme4,"description"),
			attr(demandes$probleme5,"description"),
			attr(demandes$probleme6,"description"),
			attr(demandes$probleme7,"description"),
			attr(demandes$probleme8,"description"),
			attr(demandes$probleme9,"description"),
			attr(demandes$probleme10,"description"),
			attr(demandes$probleme11,"description"),
			attr(demandes$probleme12,"description"),
			n,
			paste ("<span class=\"interet\">",format(100.0*f,nsmall=1,digits=1),"</span>", sep="")), ncol=3, dimnames=list(NULL,c("Probl&egrave;me","Fr&eacute;q.","Pourc.")))
	
	indic <- html.indicateur (
		no=7,
		descr.indicateur="Proportion des demandes concernant des usagers pr&eacute;sentant des probl&egrave;mes particuliers.",
		souspop=collectif.horsTransfert,
		tableau.special=prob,
		file=file,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	rm (temp, n, f, prob)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	
# 8

	indic <- html.indicateur(
		no=8,
		descr.indicateur="Proportion des usagers sans probl&egrave;mes mentaux ni d&eacute;ficits sur les AVQ de base, respectivement pr&eacute;sentant des d&eacute;ficits dans les AVQ, n&eacute;cessitant une surveillance, voire une aide, en raison de probl&egrave;mes mentaux.",
		souspop=collectif.horsTransfert,
		variables=list(degreavq,degrecog),
		file=file,
		seuil=seuil,
		cumul=FALSE,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
	
# 9
	indic <- html.indicateur (
		no=9,
		descr.indicateur="Distribution des aboutissements des demandes. En particulier, proportion des demandes aboutissant &agrave; un maintien &agrave; domicile.",
		souspop=collectif.horsTransfert,
		variables=demandes$aboutiss,
		file=file,
		interet=1,
		lignes.exclues=8:11,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=TRUE

	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

		
# 10

	collectif.placementCls <- detail.collectif(
		description="Demandes m&eacute;dico-sociales de placement Cls.",
		variables.souspop=list(transfer,demandes$pecdem),
		modalites.souspop=list(2,3),
		seuil=seuil
	)

	indic <- html.indicateur (
		no=10,
		descr.indicateur="Proportion des demandes de placement en lit Cls pour lesquelles le BRIO re&ccedil;oit un DMST.",
		souspop=collectif.placementCls,
		variables=demandes$dmst,
		file=file,
		interet=1,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
	
# 11
	indic <- html.indicateur(
		no=11,
		descr.indicateur="Proportion des demandes de placement en lit Cls pour lesquelles le BRIO rencontre l'usager concern&eacute; par la demande.",
		souspop=collectif.placementCls,
		variables=demandes$rencontr,
		file=file,
		interet=1,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
	
# 12

	collectif.hospitalisationA <- detail.collectif(
		description="Demandes d'orientation m&eacute;dico-sociale concernant les patients hospitalis&eacute;s en lit A.",
		variables.souspop=list(transfer,demandes$provient),
		modalites.souspop=list(2,1),
		seuil=seuil
	)

	indic <- html.indicateur(
		no=12,
		descr.indicateur="Proportion des demandes concernant des usagers hospitalis&eacute;s en h&ocirc;pital A et dont l'aboutissement est un placement en lit Cls dont il est pr&eacute;cis&eacute; qu'il s'agit d'un s&eacute;jour d'observation.",
		souspop=collectif.hospitalisationA,
		variables=demandes$aboutiss,
		file=file,
		interet=5,
		lignes.exclues=c(1:2,4,6:11),
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

# 13
	collectif.sejourObservation <- detail.collectif(
		description="Demandes d'orientation m&eacute;dico-sociale concernant les usagers plac&eacute;s en s&eacute;jour d'observation.",
		variables.souspop=list(transfer,demandes$provient),
		modalites.souspop=list(2,10),
		seuil=seuil
	)

	indic <- html.indicateur(
		no=13,
		descr.indicateur="Proportion des usagers se trouvant en s&eacute;jour d'observation r&eacute;orient&eacute;s vers une prise en charge autre qu'un placement en lit Cls.",
		souspop=collectif.sejourObservation,
		variables=demandes$aboutiss,
		lignes.ordonnees=TRUE,
		interet=c(1,2,6,7),
		lignes.exclues=c(5,8:11),
		file=file,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
	
# 14

	collectif.aboutiCls <- detail.collectif(
		description="Demandes d'orientation m&eacute;dico-sociale aboutissant &agrave; un placement en lit Cls.",
		variables.souspop=list(transfer,demandes$aboutiss),
		modalites.souspop=list(2,3),
		seuil=seuil
	)


	#jointure entre les deux fichiers

	refems <- match (x=as.character(demandes$iddemand), table=as.character(refus$iddemand)[as.integer(refus$typerefus)==2])
	refems <- ifelse (is.na(refems), 1,2)
	refems <- factor (refems, levels=1:2, labels=c("non", "oui"))
	attr (refems, "description") <- "refus EMS"
	# attention "oui" == 2 !!!


	levels (refus$motifrefus) <- html.translate (levels (refus$motifrefus))
	attr (refus$motifrefus, "description") <- "Motif du refus"
	refus.indicateur <- !is.na(match(table=demandes$iddemand[collectif.aboutiCls$indicateur&as.integer(refems)==2], x=refus$iddemand))
	tableau.special <- as.matrix(
		table(
			table(
				as.character(refus$iddemand[refus.indicateur])
			)
		)
	)

	tableau.special <- rbind (
		tableau.special,
		sum(as.integer(dimnames(tableau.special)[[1]])*tableau.special)
	)

if (is.null(dimnames(tableau.special)))	{
	dimnames(tableau.special) <- list("", "")
}
	dimnames(tableau.special)[[1]][length(dimnames(tableau.special)[[1]])] <- "Total de tous les refus"
	if (NROW(tableau.special)>0)	{
		dimnames(tableau.special)[[2]] <- c("Fr&eacute;quence par nombre de refus")


		indic <- html.indicateur(
			no=14,
			descr.indicateur="Proportion des demandes abouties sur un placement en lit Cls pour lesquelles un ou plusieurs refus d'EMS ont &eacute;t&eacute; enregistr&eacute;s de la part d'&eacute;tablissements disposant de lits vacants.",
			souspop=collectif.aboutiCls,
			variables=NULL,
			tableau.special=tableau.special,
			outside.supplement=paste (
				"<p>Proportion des demandes aboutissant &agrave; un refus : <strong>", format (100.0*length(refus$iddemand[refus.indicateur])/sum(as.integer(collectif.aboutiCls$indicateur)),nsmall=1,digits=1), "%</strong> (", length(refus$iddemand[refus.indicateur]), "/", sum(as.integer(collectif.aboutiCls$indicateur)),").</p>\n",
				"<p>Liste des motifs de refus :</p>\n",
				html.table (
					tabulate (
						x=refus$motifrefus[refus.indicateur],
						label=attr(refus$motifrefus,"description"),
						pas.cellules.vides=TRUE,
						lignes.ordonnees=TRUE
					)$table
				),
				sep=""
			),
			file=file,
			seuil=seuil,
			bavard.taux=bavard,
			bavard.tableau=bavard
		)
		reponse.faible <- c(reponse.faible, indic$var.manquantes)
		indList <- c(indList, indic$calcul.possible)

	} else	{
		if (is.null(dimnames(tableau.special))) dimnames(tableau.special) <- list(NULL,NULL)
		dimnames(tableau.special)[[2]] <- c("Fr&eacute;quence par nombre de refus")
		indic <- html.indicateur(
			no=14,
			descr.indicateur="Proportion des demandes abouties sur un placement en lit Cls pour lesquelles un ou plusieurs refus d'EMS ont &eacute;t&eacute; enregistr&eacute;s de la part d'&eacute;tablissements disposant de lits vacants (demandes d'orientation m&eacute;dico-sociale).",
			souspop=collectif.aboutiCls,
			variables=NULL,
			inside.supplement="<p>Pas de refus.</p>",
			file=file,
			seuil=seuil,
			bavard.taux=bavard,
			bavard.tableau=bavard
		)
		reponse.faible <- c(reponse.faible, indic$var.manquantes)
		indList <- c(indList, indic$calcul.possible)
	}
	rm (refems, tableau.special)
		
# 15

	indic <- html.indicateur(
		no=15,
		descr.indicateur="Proportion des demandes abouties sur un placement en lit Cls pour lesquelles le BRIO n'est qu'inform&eacute;.",
		souspop=collectif.aboutiCls,
		variables=demandes$pecdem,
		file=file,
		lignes.ordonnees=TRUE,
		interet=1,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

# 16 

	collectif.placementCls <- detail.collectif(
		description="Demandes d'orientation m&eacute;dico-sociale en lit Cls.",
		variables.souspop=list(transfer,demandes$pecdem),
		modalites.souspop=list(2,3),
		seuil=seuil
	)

	indic <- html.indicateur (
		no=16,
		descr.indicateur="Proportion des demandes de placement en <strong>lit Cls</strong> faites de fa&ccedil;on anticip&eacute;e.",
		souspop=collectif.placementCls,
		variables=demandes$debutpec,
		file=file,
		interet=2:4,
		outside.supplement="<p>Voir aussi indicateurs <a href=\"#17\">17</a>, <a href=\"#20\">20</a>, et <a href=\"#21\">21</a>.</p>",
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	
# 17 

	collectif.placementClsHorsA <- detail.collectif(
		description="Demandes de placements Cls ne concernant pas l'h&ocirc;pital A.",
		variables.souspop=list(transfer,demandes$pecdem, demandes$origine),
		modalites.souspop=list(2,3,c(1:6,8:12)),
		seuil=seuil
	)

	indic <- html.indicateur(
		no=17,
		descr.indicateur="Proportion des demandes de placement <strong>en lit Cls</strong>, faites de fa&ccedil;on anticip&eacute;e , lorsque l'origine est <strong>autre qu'un h&ocirc;pital A</strong>.",
		souspop=collectif.placementClsHorsA,
		variables=demandes$debutpec,
		tableau.special=NULL,
		file=file,
		interet=2:4,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)
		
# 18

	collectif.placementHorsCls <- detail.collectif(
		description="Demandes autres que placements Cls.",
		variables.souspop=list(transfer,demandes$pecdem),
		modalites.souspop=list(2,c(1:2,4:9)),
		seuil=seuil
	)

	indic <- html.indicateur(
		no=18,
		descr.indicateur="Proportion des demandes autres que placement en lit Cls pour lesquelles le BRIO re&ccedil;oit un document m&eacute;dico-social de transmission.",
		souspop=collectif.placementHorsCls,
		variables= demandes$dmst,
		file=file,
		interet=1,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)


# 19

	indic <- html.indicateur(
		no=19,
		descr.indicateur="Proportion des demandes autres que placement en lit Cls pour lesquelles le BRIO rencontre l'usager concern&eacute; par la demande.",
		souspop=collectif.placementHorsCls,
		variables=demandes$rencontr,
		file=file,
		interet=1,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	
# 20

	indic <- html.indicateur (
		no=20,
		descr.indicateur="Proportion des demandes <strong>autres que placement en lit Cls</strong>, faites de fa&ccedil;on anticip&eacute;e.",
		souspop=collectif.placementHorsCls,
		variables=demandes$debutpec,
		file=file,
		lignes.ordonnees=FALSE,
		interet=2:4,
		outside.supplement="<p>Voir indicateurs <a href=\"#16\">16</a>, <a href=\"#17\">17</a> et <a href=\"#21\">21</a>.</p>",
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	
# 21

	collectif.placementClsHorsA <- detail.collectif(
		description="Demandes autres que placements Cls ne concernant pas l'h&ocirc;pital A.",
		variables.souspop=list(transfer,demandes$pecdem, demandes$origine),
		modalites.souspop=list(2,c(1:2,4:9),c(1:6,8:12)),
		seuil=seuil
	)


	indic <- html.indicateur (
		no=21,
		descr.indicateur="Proportion des demandes <strong>autres que placement en lit Cls</strong>, faites de fa&ccedil;on anticip&eacute;e, pour les demandes dont l'origine est <strong>externe &agrave; l'h&ocirc;pital A</strong>.",
		souspop.description="Demandes autres que placements Cls ne concernant pas l'h&ocirc;pital A.",
		variables.souspop=list(transfer,demandes$pecdem, demandes$origine),
		modalites.souspop=list(2,c(0:2,4:9),c(1:6,8:12)),
		variables=demandes$debutpec,
		file=file,
		lignes.ordonnees=FALSE,
		interet=2:4,
		outside.supplement="<p>Voir indicateurs <a href=\"#16\">16</a>, <a href=\"#17\">17</a> et <a href=\"#20\">20</a>.</p>",
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

		
# 22
	# regroupement en catgories
	toto <- rep(NA, length(demandes$demabout))
	toto[demandes$demabout>=0 & demandes$demabout<=2] <- 1
	toto[demandes$demabout>=3 & demandes$demabout<=6] <- 2
	toto[demandes$demabout>=7 & demandes$demabout<=30] <- 3
	toto[demandes$demabout>=31] <- 4
	toto[demandes$demabout<0] <- 0
	toto <- factor (toto, levels=0:4, labels=c("Aboutissement avant transmission ou erreur de saisie", "0-2 jours", "3-6 jours", "7-30 jours", "31 jours et plus"))
	attr(toto, "description") <- "Dur&eacute;e en jours entre la date de la demande et la date d'aboutissement"
	indic <- html.indicateur (
		no=22,
		descr.indicateur="Proportion des demandes abouties dans les ... jours suivant la demande, en fonction du d&eacute;but de la prise en charge d&eacute;sir&eacute;e.",
#		souspop=collectif.placementHorsCls,
		souspop=collectif.horsTransfert,
		variables=list(demandes$debutpec, toto),
		file=file,
		seuil=seuil,
		cumul=TRUE,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

			
# 23

	indic <- html.indicateur (
		no=23,
		descr.indicateur="Proportion des demandes en cours de traitement au terme de la p&eacute;riode d'observation (toutes demandes confondues).",
		variables=transfer,
		file=file,
		interet=3,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

			
# 24

	indic <- html.indicateur (
		no=24,
		descr.indicateur="Proportion des demandes concernant des usagers dont le domicile est couvert par un autre r&eacute;seau.",
		souspop=collectif.horsTransfert,
		variables=regres,
		file=file,
		interet=2,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

		
# 25

	indic <- html.indicateur (
		no=25,
		descr.indicateur="Proportion des demandes relatives aux usagers couverts par le r&eacute;seau dont l'aboutissement implique les structures d'un autre r&eacute;seau.",
#		souspop=collectif.regionReseau,
		souspop.description="Demandes d'orientation m&eacute;dico-sociale provenant du r&eacute;seau et aboutissant en EMS",
		variables.souspop=list(transfer,demandes$aboutis, regres),
		modalites.souspop=list(2,2:5,1),
		variables=demandes$emsres,
		tableau.special=NULL,
		file=file,
		interet=1,
		seuil=seuil,
		bavard.taux=bavard,
		bavard.tableau=bavard
	)
	reponse.faible <- c(reponse.faible, indic$var.manquantes)
	indList <- c(indList, indic$calcul.possible)

	if (!is.null(reponse.faible) && length(reponse.faible)>0)	{
		reponse.faible <- unlist(reponse.faible)
		reponse.faible <- sub (x=reponse.faible, pattern=attr(transfer,"description"), replacement=attr(demandes$provient,"description"), extended=FALSE)
		cat (
			"<hr /><h3>Liste des variables pour lesquelles le taux de r&eacute;ponse est insuffisant</h3>\n",
			html.ul(sort(unique(reponse.faible))),
			file=file,
			append=TRUE
		)
		html.p ("<strong>Attention :</strong> le taux de r&eacute;ponse des variables qui n'entrent pas individuellement dans le calcul d'un indicateur doit parfois d&eacute;passer de beaucoup le seuil limite pour que la combinaison de celles-ci l'atteigne. Dans la liste peuvent figurer des variables dont le taux de r&eacute;ponse <strong>individuel</strong> est suffisant, mais plus apr&egrave;s <strong>combinaison</strong>.", file=file)
		indList <- ifelse(matrix(indList,nrow=1), "", "X")
		dimnames(indList) <- list(
			brio,
			paste ("<a href=\"#", c(1:4,rep(5,5),6:25), "\">", c(1:4, "5a", "5b", "5c", "5d", "5e", 6:25), "</a>", sep="")
		)
		cat ("<h3>R&eacute;sum&eacute; des indicateurs qui n'ont pu &ecirc;tre calcul&eacute;s</h3>\n", file=file, append=TRUE)
		cat (html.table (indList), file=file, append=TRUE)
		html.p ("\"X\" : taux de r&eacute;ponse insuffisant.", file=file)
	}
# Fin

	html.end (file=file)
	indList
}
