<?php
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
/**
* Ensemble de donnes pour l'extraction des statistiques par EMS
* @package donnes
*/
/**
* Ensemble de donnes pour l'extraction des statistiques par EMS
* Groupe les requtes de base de donnes ncessaires  l'extraction des statistiques par EMS
* et rend disponibles les indicateurs pour affichage ou pour export dans une feuille de calcul
* @package donnes
* @author ob
*/
class ensStatEMS extends connexionBrioche {
  /**
  * le numro de l'tablissement courant
  * @var int
  */
  var $ems;
  
  /**
  * le nom de l'tablissement courant
  * @var string
  */
  var $nomems;
  
  /**
  * nombre d'annonces de lit faites par l'tablissement
  * @var int
  */
  var $nbann;
  
  /**
  * nombre de lits attribus par le BRIO dans l'tablissement
  * @var int
  */
  var $nbattr;
  
  /**
  * nombre de lits de long sjour attribus par le BRIO dans l'tablissement
  * @var int
  */
  var $nbattrLong;
  
  /**
  * nombre de lits attribus par le BRIO dans l'tablissement pour un sjour d'observation
  * @var int
  */
  var $nbattrObs;
  
  /**
  * nombre de lits de court sjour (hors planning) attribus par le BRIO dans l'tablissement
  * @var int
  */
  var $nbattrCourt;
  
  /**
  * nombre de lits de court sjour "Planning" attribus par le BRIO dans l'tablissement
  * @var int
  */
  var $nbattrPlan;
  
  /**
  * le dlai moyen entre l'annonce et l'attribution du lit
  * @var float
  */
  var $delai;
  
  /**
  * le dlai mdian entre l'annonce et l'attribution du lit
  * @var int
  */
  var $percent;
  
  /**
  * le dlai moyen entre l'annonce et l'attribution du lit pour les courts sjours (planning except)
  * @var float 
  */
  var $delaiCourt;
  
  /**
  * le dlai mdian entre l'annonce et l'attribution du lit pour les courts sjours (planning except)
  * @var int
  */
  var $percentCourt;
  
  /**
  * le nombre total d'annonce reues par le BRIO (pour comparaison)
  * @var int
  */
  var $nbannBrio;
  
  /**
  * le nombre total d'attributions faites par le BRIO (pour comparaison)
  * @var int
  */
  var $nbattrBrio;
  
  /**
  * le nombre total d'attributions "long sjour" faites par le BRIO (pour comparaison)
  * @var int
  */
  var $nbattrLongBrio;
  
  /**
  * le nombre total d'attributions "sjour d'observation" faites par le BRIO (pour comparaison)
  * @var int
  */
  var $nbattrObsBrio;
  
  /**
  * le nombre total d'attribution "court sjour" hors planning faites par le BRIO (pour comparaison)
  * @var int
  */
  var $nbattrCourtBrio;
  
  /**
  * le nombre total d'attribution "court sjour planning" faites par le BRIO (pour comparaison)
  * @var int
  */
  var $nbattrPlanBrio;
  
  /**
  * le nombre d'annonces de long sjour par l'tablissement
  * @var int
  */
  var $nbannLong;
  
  /**
  * le nombre total d'annonces de long sjour reues par le BRIO (pour comparaison)
  * @var int
  */
  var $nbannLongBrio;
  
  /**
  * le nombre d'annonces de court sjour (hors planning) faites par l'tablissement
  * @var int
  */
  var $nbannCourt;
  
  /**
  * le nombre total d'annonces de court sjour (hors planning) reues par le BRIO (pour comparaison)
  * @var int
  */
  var $nbannCourtBrio;
  
  /**
  * le nombre d'annonces de lits polyvalents faites par l'tablissement
  * @var int
  */
  var $nbannPoly;
  
  /**
  * le nombre total d'annonces de lit polyvalents reus par le BRIO (pour comparaison)
  * @var int
  */
  var $nbannPolyBrio;
  
  /**
  * le nombre courts sjours "planning" dans l'tablissement
  * @var int
  */
  var $nbannPlan;
  
  /**
  * le nombre total de courts sjours "panning" organiss par le BRIO (pour comparaison)
  * @var int
  */
  var $nbannPlanBrio;
  
  /**
  * le nombre d'annonces de lit retires par l'tablissement
  * @var int
  */
  var $nbRetr;
  
  /**
  * le nombre total de retraits d'annonces de lits reus par le BRIO
  * @var int
  */
  var $nbRetrBrio;
  
  /**
  * le premier quartile du dlai d'attribution des lits non court-sjour par le BRIO (tous tablissements)
  * @var int
  */
  var $percent25BrioNonCourt;
  
  /**
  * la mdiane du dlai d'attritution des lits non court-sjour par le BRIO (tous tablissements)
  * @var int
  */
  var $percent50BrioNonCourt;
  
  /**
  * le troisime quartile du dlai d'attribution des lits non court-sjour par le BRIO (tous tablissements)
  * @var int
  */
  var $percent75BrioNonCourt;
  
  /**
  * la moyenne d'ge des usagers concerns par l'tablissement
  * @var float
  */
  var $moyenneDage;
  
  /**
  * la moyenne d'ge globale des usagers concerns (BRIO entier)
  * @var float
  */
  var $moyenneDageBrio;
  
  /**
  * le nombre de femmes concernes par l'tablissement
  * @var int
  */
  var $nbFemmes;
  
  /**
  * le nombre de femmes concernes (BRIO entier)
  * @var int
  */
  var $nbFemmesBrio;
  
  /**
  * le nombre d'hommes concerns par l'tablissement
  * @var int
  */
  var $nbHommes;
  
  /**
  * le nombre d'hommes concerns (BRIO entier)
  * @var int
  */
  var $nbHommesBrio;
  
  /**
  * le dbut de la priode concerne (date au format "francophone")
  * @var string
  */
  var $deb;
  
  /**
  * la fin de la priode concerne (date au format "francophone")
  * @var string
  */
  var $fin;

  /**
  * @param string deb cf. variable de classe
  * @param string fin cf. variable de classe
  * @param string sessionid cf. variable de classe
  * @param int ems cf. variable de classe
  * @param array autspec les autorisations spcifiques
  */
  function ensStatEMS ($deb, $fin, $sessionid, $ems = "", $autspec = 0){
    $ini = new briocheIni();
    $autspec[0] = $ini->autSpec[0];
    if (! $this->connexionBrioche($sessionid, $autspec)) return;
    // L'autorisation d'accs aux dossiers Brioche est absente, il s'agit donc d'un utilisateur EMS
    // qui n'accde qu' l'tablissement qui lui a t attribu
    if (! $this->verifAut($ini->autReq, $this->autorisations)){
      $eu = new enrUtilisateur($this->idsession, $this->autspec);
      $ett = $eu->getEtablAttribue("ems");
      $this->ems = $ett->noet;
      $this->nomems = $ett->nomet;
    }
    else { // utilisateur "BRIO", disposant du droit d'accs aux dossiers
      $this->ems = $ems;
      $et = new ensTableBrioche("etablissements", "noet = $this->ems", "nomet", 0, $this->idsession);
      $ett = $et->getTableau();
      if (count($ett)) $this->nomems = $ett[0]->nomet;
    }
    $debAm = $this->getDateAm($deb);
    $finAm = $this->getDateAm($fin);
    $this->deb = $deb;
    $this->fin = $fin;
    //Annonces de lit
    $ann = new ensTableBrioche("annoncelit", "", "annoncelit.nolit", 0, $this->idsession, "", "", false, $autspec);
    
    //Attributions
    $attr = new ensTableBrioche("attributionlit", "", "attributionlit.nolit", 0, $this->idsession, "annoncelit",
                                "attributionlit.nolit = annoncelit.nolit", false, $autspec);
    
    // Usagers
    $join[] = "dossiers";
    $join[] = "usagers";
    $join[] = "annoncelit";
    $on[] = "attributionlit.nodoss = dossiers.nodoss";
    $on[] = "dossiers.nous = usagers.nous";
    $on[] = "attributionlit.nolit = annoncelit.nolit";
        
    $doss = new ensTableBrioche("attributionlit", "", "attributionlit.nolit", 0, $this->idsession, $join, $on, false, $autspec);

    // Annonces
    $wherebrio = "datann BETWEEN $debAm AND $finAm";
    $wheregen = $wherebrio  . ($this->ems == -1 ? "" : " AND annoncelit.etabl = $this->ems");
    
    $wherepart = " AND stlit = 1"; // Annonces pour longs sjours
    $ann->setWhere($wherebrio);
    $this->nbannBrio = $ann->compte();
    $ann->setWhere($wherebrio . $wherepart);
    $this->nbannLongBrio = $ann->compte();
    $ann->setWhere($wheregen);
    $this->nbann = $ann->compte();
    $ann->setWHere($wheregen . $wherepart);
    $this->nbannLong = $ann->compte();
    
    
    $wherepart = " AND stlit = 3"; // Courts sjours (sans planning)
    $ann->setWhere($wherebrio . $wherepart);
    $this->nbannCourtBrio = $ann->compte();
    $ann->setWhere($wheregen . $wherepart);
    $this->nbannCourt = $ann->compte();
    
    $wherepart = " AND stlit NOT IN(1, 3, 5)"; // Polyvalents (= tout le reste)
    $ann->setWhere($wherebrio . $wherepart);
    $this->nbannPolyBrio = $ann->compte();
    $ann->setWhere($wheregen . $wherepart);
    $this->nbannPoly = $ann->compte();
    
    $wherepart = " AND stlit = 5"; // Planning
    $ann->setWhere($wherebrio . $wherepart);
    $this->nbannPlanBrio = $ann->compte();
    $ann->setWhere($wheregen . $wherepart);
    $this->nbannPlan = $ann->compte();

    
    
    // Attributions
    $wherebrio = "dateplac BETWEEN $debAm AND $finAm AND typesej <> 99 AND attributionlit.nolit IS NOT NULL";
    $wheregen = $wherebrio . ($this->ems == -1 ? "" : " AND etabl = $this->ems");
    $attr->setWhere($wheregen);
    $this->nbattr = $attr->compte();
    $attr->setWhere($wherebrio);
    $this->nbattrBrio = $attr->compte();
    
    $wherepart = " AND typesej = 1";
    $attr->setWhere($wheregen . $wherepart);
    $this->nbattrLong = $attr->compte();
    $attr->setWhere($wherebrio . $wherepart);
    $this->nbattrLongBrio = $attr->compte();
    $wherepart = " AND typesej = 2";
    $attr->setWhere($wheregen . $wherepart);
    $this->nbattrObs = $attr->compte();
    $attr->setWhere($wherebrio . $wherepart);
    $this->nbattrObsBrio = $attr->compte();
    $wherepart = " AND typesej = 3";
    $attr->setWhere($wheregen . $wherepart);
    $this->nbattrCourt = $attr->compte();
    $attr->setWhere($wherebrio . $wherepart);
    $this->nbattrCourtBrio = $attr->compte();
    $attr->setWhere($wheregen);
    $wherepart = " AND typesej = 5";
    $attr->setWhere($wheregen . $wherepart);
    $this->nbattrPlan = $attr->compte();
    $attr->setWhere($wherebrio . $wherepart);
    $this->nbattrPlanBrio = $attr->compte();
    
    $whereretrait = "dateplac BETWEEN $debAm AND $finAm AND annoncelit.nolit IS NOT NULL";
    $wherepart = " AND typesej NOT IN (1, 2, 3, 5)";
    $wheregen = $whereretrait . ($this->ems == -1 ? "" : " AND etabl = $this->ems");
    $attr->setWhere($wheregen . $wherepart);
    $this->nbRetr = $attr->compte();
    $attr->setWhere($whereretrait . $wherepart);
    $this->nbRetrBrio = $attr->compte();
    
    $wheregen = $wherebrio . ($this->ems == -1 ? "" : " AND etabl = $this->ems");
    $wherepart = " AND typesej IN (1, 2)";
    $attr->setWhere($wheregen . $wherepart);
    $this->delai = $attr->moyenne("TO_DAYS(dateplac) -  TO_DAYS(litdepuis)");
    $this->percent = $attr->percentile("TO_DAYS(dateplac) - TO_DAYS(litdepuis)", 50);
    
    $wherepart = " AND typesej = 3";
    $attr->setWhere($wheregen . $wherepart);
    $this->delaiCourt = $attr->moyenne("TO_DAYS(dateplac) -  TO_DAYS(litdepuis)");
    $this->percentCourt = $attr->percentile("TO_DAYS(dateplac) - TO_DAYS(litdepuis)", 50);
    
    
    $wherepart = " AND typesej NOT IN (3, 5)";
    $attr->setWhere($wherebrio . $wherepart);
    $this->percent25BrioNonCourt = $attr->percentile("TO_DAYS(dateplac) - TO_DAYS(litdepuis)", 25);
    $this->percent50BrioNonCourt = $attr->percentile("TO_DAYS(dateplac) - TO_DAYS(litdepuis)", 50);
    $this->percent75BrioNonCourt = $attr->percentile("TO_DAYS(dateplac) - TO_DAYS(litdepuis)", 75);

    
    
    
    //Usagers
    $doss->setWhere($wheregen);
    $this->moyenneDage = $doss->moyenne("YEAR(attributionlit.dateplac) - YEAR(usagers.datenaiss)");
    $doss->setWhere($wherebrio);
    $this->moyenneDageBrio = $doss->moyenne("YEAR(attributionlit.dateplac) - YEAR(usagers.datenaiss)");
    $wherepart = " AND sexe = \"F\"";
    $doss->setWhere($wheregen . $wherepart);
    $this->nbFemmes = $doss->compte();
    $doss->setWhere($wherebrio . $wherepart);
    $this->nbFemmesBrio = $doss->compte();
    $wherepart = " AND sexe = \"M\"";
    $doss->setWhere($wheregen . $wherepart);
    $this->nbHommes = $doss->compte();
    $doss->setWhere($wherebrio . $wherepart);
    $this->nbHommesBrio = $doss->compte();
  }

  /**
  * renvoie un tableau bidimensionnel qui peut servir de substrat  une feuille de calcul
  * @return array
  */
  function getTableau() {
    $tab[0][0] = "<H2>Etablissement : $this->nomems</H2>";
    $tab[1][0] = "Priode du $this->deb au $this->fin";
    $tab[2][0] = "Nombre de lits annoncs au BRIO";
    $tab[2][2] = $this->nbann;
    $tab[3][0] = "<I>dont lits de long sjour</I>";
    $tab[3][1] = $this->nbannLong;
    $tab[4][0] = "<I>dont lits de court sjour</I>";
    $tab[4][1] = $this->nbannCourt;
    $tab[5][0] = "<I>dont lits polyvalents</I>";
    $tab[5][1] = $this->nbannPoly;
    $tab[6][0] = "<I>dont court sjour \"planning\"</I>";
    $tab[6][1] = $this->nbannPlan;
   
    
    $tab[7][0] = "Nombre de lits attribus";
    $tab[7][2] =  $this->nbattr;
    $tab[8][0] = "<I>dont longs sjours</I>";
    $tab[8][1] = $this->nbattrLong;
    $tab[9][0] = "<I>dont sjours d'observation</I>";
    $tab[9][1] = $this->nbattrObs;
    $tab[10][0] = "<I>dont courts sjours (sans planning)</I>";
    $tab[10][1] = $this->nbattrCourt;
    $tab[11][0] = "<I>dont courts sjours \"planning\"</I>";
    $tab[11][1] = $this->nbattrPlan;
    
    $tab[12][0] = "Nombre d'annonces de lit retires";
    $tab[12][1] = $this->nbRetrBrio;
    
    $tab[13][0] = "Dlai moyen disponibilit-attribution (jours) - longs sj., y compris sj. obs.";
    $tab[13][2] = $this->delai;
    $tab[14][0] = "Dans la moiti des cas, dlai infrieur  (jours)";
    $tab[14][2] = $this->percent;
    
    
    $tab[15][0] = "Dlai moyen disponibilit-attribution (jours) - courts sj., sans planning";
    $tab[15][2] = $this->delaiCourt;
    $tab[16][0] = "Dans la moiti des cas, dlai infrieur  (jours)";
    $tab[16][2] = $this->percentCourt;
    
    
    $tab[17][0] = "Moyenne d'ge des usagers (ans)";
    $tab[17][2] = $this->moyenneDage;
    $tab[18][0] = "Rapport femmes/hommes";
    $tab[18][2] = ($this->nbHommes ? round($this->nbFemmes / $this->nbHommes, 2) : "infini");

    $tab[19][3] = "Donnes extraites le " . date("d.m.Y") . " par $this->nomplein";
    return $tab;
  }
}
?>
