<?php
 // $Id: ensTableBrioche.inc,v 1.3 2017/07/06 22:19:44 ob Exp $
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
/**
* Ensemble permettant d'accder  une table ou une jointure
* @package donnes
*/
/**
* Ensemble permettant d'accder  une table ou une jointure
* Cet objet encapsule la plupart des requtes slection dont l'application a besoin et permet
* d'obtenir des tableaux de donnes. Il permet d'ignorer les enregistrements provisoires ( dtruire 
* par les scripts de maintenance), de crer de nouveaux enregistrement lorsqu'il faut connatre leur
* identifiant et de dtruire un enregistrement. Il ne prend pas en charge les requtes mise--jour,
* qui se font par l'entremise de updateTableBrioche
* <pre>
* $et = new ensTableBrioche("nomtable", "1", "*", 0, $sessionid);
* $tet = $et->getTableau();
* print $tet[0]->nomchamp
* </pre>
* @package donnes
* @author ob
*/
class ensTableBrioche extends connexionBrioche{
  /**
  * le nom de la table (ou de la table principale dans le cas des jointures)
  * @var string
  */
  var $table;
  
  /**
  * la clause WHERE
  * @var string
  */
  var $where;
  
  /**
  * la liste des champs slectionns (spars par une virgule)
  * @var string
  */
  var $champs;
  
  /**
  * la valeur du tst des enregistrements provisoires ( supprimer) : la suppression se fait toujours en deux fois.
  * lorsqu'un enregistrement est cr, il porte le timestamp 0 puis, s'il n'est pas valid, il prend la valeur
  * de t1 lors d'une premire excution du script de maintenance et est dtruit lors de l'excution suivante.
  * La valeur de t1 est dtermine par le paramtre tstorig de briocheIni.inc. Tous les enregistrements portant
  * ce timestamp sont ignors 
  * @var int
  */
  var $t1;
  
  /**
  * les champs de la clause ORDER (spars par une virgule)
  * @var string
  */
  var $order;
  
  /**
  * la clause JOIN : s'il n'y a qu'une jointure, string nom de la table ; s'il y en a plusieurs, un tableau des noms de table
  * @var mixed
  */
  var $join;
 
  /**
  * la clause ON : s'il n'y a qu'une jointure, string "table1.champ1 = table2.champ2" ; s'il y en a plusieurs, un tableau
  * les index des tableaux $join et $on doivent concider
  * @var mixed
  */
  var $on;
  /**
  * s'agit-il d'une requte "SELECT DISTINCT" ?
  * @var boolean
  */
  var $distinct;
  
  /**
  * le texte complet de la requte sql
  * @var string
  */
  var $sql;
  
  /**
  * la clause GROUP
  * @var string
  */
  var $group;
  
  /**
  * ventuellement, le nombre limite (correspondant  la clause LIMIT)
  * @var int
  */
  var $limit;
  
  /**
  * ventuellement, l'offset de la clause LIMIT
  * @var int
  */
  var $offset;
  
  /**
  * une ventuelle clause HAVING
  * @var string
  */
  var $having;
  
  /**
  * @param string table cf. variable de classe
  * @param string where cf. variable de classe
  * @param string champs cf. variable de classe
  * @param string order cf. variable de classe
  * @param string sessionid l'identifiant de session
  * @param mixed join cf. variable de classe
  * @param mixed on cf. variable de classe
  * @param boolean distinct cf. variable de classe
  * @param array autspec les autorisations spcifiques
  * @param string group cf. variable de classe
  * @param int limit cf. variable de classe
  * @param int offset cf. variable de classe
  * @param string having cf. variable de classe
  */ 
  function __construct($table, $where="1", $champs="*", $order=0, $sessionid,
                           $join=0, $on=0, $distinct=false, $autspec=0, $group="", $limit = 0,
                           $offset = 0, $having=""){
    if (! parent::__construct($sessionid, $autspec)) return false;
    $this->table = $table;
    $this->where = $where;
    $this->champs = $champs;
    $this->order = $order;
    $this->join = $join;
    $this->on = $on;
    $this->distinct = $distinct;
    $this->group = $group;
    $this->limit = $limit;
    $this->offset = $offset;
    $this->having = $having;
    $ini = new briocheIni();
    $this->t1 = $ini->tstorig;   //timestamp des enregistrements  supprimer
    return true;
  }

  /**
  * renvoie un tableau bidimensionnel des donnes utilisable pour une feuille de calcul
  * @param boolean masqueTech des champs techniques, commenant par "_", peuvent si souhait tre ignors
  * @return array un tableau bidimensionnel
  */
  function getTableauBidim($masqueTech = false){
    $requete = $this->getSql();
    $tab = array();
    $result = mysqli_query($this->linkBrioche, $requete);
    if ($result){
      //$nf = mysql_num_fields($result);
      $masque = array();
      //for ($i = 0; $i < $nf; $i++){
      while ($fldd = mysqli_fetch_field($result)){
        if ((! $masqueTech) || (! (substr($fldd->name, 0, 1) == "_"))) {
        //Les champs "techniques" dont le nom commence par _ peuvent tre masqus
          $types[] = $fldd->type;
          $noms[] = "<B>$fldd->name</B>";
        }
        else {
          $masque[] = $i;
        }
      }
      $tab[] = $noms;
      while ($ligne = mysqli_fetch_row($result)){
        //Ablation des champs techniques
        $cptc = count($masque);
        for ($k = $cptc-1 ; $k >= 0; $k --){
          $ligne1 = array_slice($ligne, 0, $masque[$k]);
          $ligne2 = array_slice($ligne, $masque[$k] + 1);
          $ligne = array_merge($ligne1, $ligne2);
        }
        //Conversion des champs date
        for ($i = 0; $i < $nf-count($masque); $i++){
          if ($types[$i] == "date"){
            $ligne[$i] = $this->getDateFr($ligne[$i]);
          }
          $ligne[$i] = $this->metEnFormeSel($ligne[$i]);
        }
        $tab[] = $ligne;
      }
    }
    return $tab;
  }

  /**
  * renvoie le texte complet de la requte sql
  * @param boolean compte s'agit-il d'une requte "COUNT" ?
  * @param boolean limite s'agit-il d'une requte avec "LIMIT" ?
  */
  function getSql($compte = false, $limite = true){
    if ($compte) {
      $requete = "SELECT COUNT(*) ";
    }  
    else {
      $requete = "SELECT " . ($this->distinct ? "DISTINCT " : "") . "$this->champs ";
    }
    $requete .= "FROM $this->table ";
    if ($this->join && $this->on){
      if (! is_array($this->join)){
        $cpt = 1;
      }
      else {
        $cpt = count($this->join);
      }
      if($cpt == 1){
        $requete .= "LEFT JOIN $this->join ON $this->on ";
      }
      else {
        for($i = 0; $i < $cpt; $i++){
          $requete .= "LEFT JOIN " . $this->join[$i] . " ON " . $this->on[$i] . " ";
        }
      }
    }
    
    $tables = explode(',',$this->table);
       
    $requete .= "WHERE ";
    foreach ($tables as $table) {
      $requete .= "($table.tst > $this->t1) AND ";
    }
    
    //$requete .= "WHERE ($this->table.tst > $this->t1 ) AND ($this->where)" . 
    $requete .= "($this->where)" .
    ($this->group ? " GROUP BY $this->group" : "") .
    ($this->having ? " HAVING $this->having" : "") . 
    ($this->order && ! $compte ? " ORDER BY $this->order" : "") .
    ($limite && ($this->limit || $this->offset) ? " LIMIT " . ($this->offset ? $this->offset : "0") .
    ", $this->limit" : "");
    // print $requete . "<BR>\n\n";
    return $requete;
  }

  /**
  * nombre total d'enregistrements renvoys par la requte
  * @return int
  */
  function compteTotal(){
    $req = $this->getSql(1, 0);
    $result = mysqli_query($this->linkBrioche, $req);
    if ($result){
      $ligne = mysqli_fetch_row($result);
      return $ligne[0];
    }
    else return 0;
  }
  
  /**
  * renvoie un tableau comprenant un objet par index, chaque objet reprsentant une ligne du rsultat de la requte.
  * Les proprits de ces objets portent les nom des champs de la requte
  * (v. les noms dfinis par une clause AS) et contiennent la valeur correspondante
  * @param array recup un tableau de valeurs postes dans un formulaire si l'on veut shunter le recours  la base de donnes lors de la
  * vrification des donnes saisies
  * @return array le tableau d'objets correspondant au rsultat de la requte
  */
  function getTableau($recup=0){
    if ($recup) { //On rcupre simplement le tableau qui a t pass (courcircuitage de la base de donnes)
      $tab[] = $this->faitRecup($recup, $this->table);
      return $tab;
    }
    $requete = $this->getSql();
    //print $requete;
    return $this->executeReq($requete);
  }

  /**
  * Excution d'une requte
  * @param string req le texte SQL  excuter
  * @return array un tableau d'objets (cf. fonction getTableau)
  */
  function executeReq($req){
    $tab = array();
    //print "R : " . $req . "<BR>";
    $result = mysqli_query($this->linkBrioche, $req);
    //print_r($result);
    //print mysql_error();
    $tabtypes = array();
    if ($result){
      while($fld = mysqli_fetch_field($result)){
        //print $fld->name . " - " . $fld->type . "<BR>";
        $tabtypes[$fld->name] = $fld->type;
      }
      while ($ligne = mysqli_fetch_assoc($result)){
        $obj = new stdClass;
        foreach($ligne as $cle => $valeur){
          if ($tabtypes[$cle] == 7){ //timestamp
            $ligne[$cle] = date("d.m.Y H:i:s",strtotime($ligne[$cle]));
          }
          elseif ($tabtypes[$cle] == 10  // date
                OR preg_match("/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/", $ligne[$cle])
                OR $tabtypes[$cle] == 12 //datetime
                OR preg_match("/^[0-9]{4}-[0-9]{2}-[0-9]{2}\s[0-9]{2}:[0-9]{2}:[0-9]{2}$/", $ligne[$cle])){
               
            $ligne[$cle] = $this->getDateFr($ligne[$cle]);        
            
          }
          $ligne[$cle] = $this->metEnFormeSel($ligne[$cle]);
          $obj->$cle = $ligne[$cle];
        //print "<br>";
        //Conversion des champs date
        //for ($i = 0; $i < $nf; $i++){
        //while($flds = mysqli_fetch_field($result)){
        //  $fn = $flds->name;
        //  if ($flds->type == 7) { // timestamp (7)
        //    $ligne->$fn = date("d.m.Y H:i:s",strtotime($ligne->$fn));
        //  } elseif ($flds->type == 10 OR
        //      preg_match("/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/", $ligne->$fn) OR 
	//          $flds->type == 12 OR
	//          preg_match("/^[0-9]{4}-[0-9]{2}-[0-9]{2}\s[0-9]{2}:[0-9]{2}:[0-9]{2}$/", $ligne->$fn)){ //date (10), datetime (12)
                //print $ligne->$fn . "<br>";
	//        $ligne->$fn = $this->getDateFr($ligne->$fn);
        //$ligne->$fn = $this->metEnFormeSel($ligne->$fn);
        }
               
        $tab[] = $obj;
      }
    }
    //print_r($tab) . "<BR>";
    return $tab;
  }
  
  
  /**
  * modification des clauses join et on
  * @param mixed la nouvelle clause join (string ou array)
  * @param mixed la clause on correspondante (string ou array)
  */
  function setJoinOn($join, $on){
    $this->join = $join;
    $this->on = $on;	
  }	
  
  /**
  * modification de la clause where
  * @param string where la nouvelle clause
  */
  function setWhere($where){
    $this->where = $where;
  }
  
  /**
  * modification de la table principale
  * @param string le nom de la nouvelle table
  */
  function setTable($table){
    $this->table = $table;
  }

  /**
  * modification de la liste des champs slectionns
  * @param string la liste des champs (spars par une virgule)
  */
  function setChamps($champs){
    $this->champs = $champs;
  }
  
  /**
  * modification de la clause order
  * @param string la nouvelle clause
  */
  function setOrder($order){
    $this->order = $order;
  }
  
  /**
  * modification de la limite
  * @param int limite la nouvelle limite (nb. d'enregistrement)
  */
  function setLimit($limit){
    $this->limit = $limit;
  }
  
  /**
  * modification de l'offset (si limite)
  * @param int le nouvel offset (le premier enregistrement d'une requte limite)
  */
  function setOffset($offset){
    $this->offset = $offset;
  } 
  
  /**
  * Coup d'oeil  la valeur d'un champ (utilisable seulement s'il n'y a pas de jointure)
  * @param string champ le champ  lire
  * @param string where la clause where (s'il y a plusieurs enregistrements concerns, seul le premier est lu)
  * @return mixed la valeur recherche
  */
  function lookUp($champ, $where){
    $requete = "SELECT $champ FROM $this->table WHERE ($this->table.tst > $this->t1) AND ($where)";
    //print $requete;
    $result = mysqli_query($this->linkBrioche, $requete);
    if (! $result) return false;
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }
   
  /**
  * le premier enregistrement d'une table, en considrant un champ donn (utilisable uniquement s'il n'y a pas de jointure)
  * @param string champ le nom du champ
  * @return mixed la premire valeur dans l'ordre de tri
  */
  function getPremier($champ){
    $requete = "SELECT $champ FROM $this->table " .
    "WHERE ($this->where) AND ($this->table.tst > $this->t1) ORDER BY $champ LIMIT 1";
    $result = mysqli_query($this->linkBrioche, $requete);
    if(! $result) return 0;
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }

  /**
  * le dernier enregistrement d'une table, en considrant un champ donn (utilisable uniquement s'il n'y a pas de jointure)
  * @param string champ le nom du champ
  * @return mixed la dernire valeur dans l'ordre de tri
  */
  function getDernier($champ){
    $requete = "SELECT $champ FROM $this->table " .
    "WHERE ($this->where) AND ($this->table.tst > $this->t1) ORDER BY $champ DESC LIMIT 1";
    $result = mysqli_query($this->linkBrioche, $requete);
    if(! $result) return 0;
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }

   /**
  * l'enregistrement suivant d'une table, en considrant un champ donn (utilisable uniquement s'il n'y a pas de jointure)
  * @param string champ le nom du champ
  * @return mixed la valeur de l'enregisgrement suivant dans l'ordre de tri
  */
  function getSuivant($item, $champ){
    if (! $item) $item = "0";
    $requete = "SELECT $champ FROM $this->table " .
    ($this->join && $this->on ? "LEFT JOIN $this->join ON $this->on " : "") .
    "WHERE ($this->where) AND ($this->table.tst > $this->t1) AND ($champ > $item) ORDER BY $champ LIMIT 1";
    $result = mysqli_query($this->linkBrioche, $requete);
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }

   /**
  * l'enregistrement prcdent d'une table, en considrant un champ donn (utilisable uniquement s'il n'y a pas de jointure)
  * @param string champ le nom du champ
  * @return mixed la valeur de l'enregisgrement prcdent dans l'ordre de tri
  */
  function getPrecedent($item, $champ){
    if (!$item) $item = 100000000000;
    $requete = "SELECT $champ FROM $this->table " .
    ($this->join && $this->on ? "LEFT JOIN $this->join ON $this->on " : "") .
    "WHERE ($this->where) AND ($this->table.tst > $this->t1) AND ($champ < $item) ORDER BY $champ DESC LIMIT 1";
    $result = mysqli_query($this->linkBrioche, $requete);
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }

  /**
  * Contrle la prsence d'un enregistrement portant un identifiant numrique donn (utilisable uniquement s'il n'y a pas de jointure)
  * @param int la valeur de l'identifiant
  * @param string le nom du champ concern
  * @return resource le rsultat d'une requte ou false si l'enregistrement n'existe pas
  */
  function checkItem($item, $champ){
    $requete = "SELECT COUNT(*) FROM $this->table " .
    ($this->join && $this->on ? "LEFT JOIN $this->join ON $this->on " : "") .
    "WHERE $champ = $item";
    $result = mysqli_query($this->linkBrioche, $requete);
    if (! $result) return 0;
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }

  /**
  * cre un nouvel enregistrement dans une table et renvoie son identifiant
  * la table doit comporter un champ intitul "dmd5" pouvant contenir un hchage md5
  * @param string le nom du champ qui sert d'identifiant
  * @return int le numro de l'identifiant du nouvel enregistrement
  */
  function creeNouveau($champ){
    $dg = md5(time(). $this->idsession);
    $requete = "INSERT INTO $this->table SET dmd5 = \"$dg\", tst = 0";
    mysqli_query($this->linkBrioche, $requete);
    $requete = "SELECT $champ FROM $this->table WHERE dmd5 = \"$dg\"";
    $result = mysqli_query($this->linkBrioche, $requete);
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }

  /**
  * dtruit un enregistrement de la table $this->table
  * @param mixed item un identifiant d'enregistrement
  * @param string champ le nom du champ correspondant
  * @param boolean numerique s'agit-il d'un champ numrique ?
  */
  function detruitEnr($item, $champ, $numerique=true){
    $requete = "DELETE FROM $this->table WHERE $champ = " . ($numerique ? "$item" : "\"$item\"");
    // print $requete;
    return mysqli_query($this->linkBrioche, $requete);
  }
  
  /**
  * dtruit la totalit des enregistrements rpondant  la clause WHERE (prudence !)
  * @return resource le rsultat de la requte destruction
  */
  function detruitEnsemble(){
    if ($this->join) return false;
    $requete = "DELETE FROM $this->table WHERE $this->where";
    //print  "RRRRRRRRR $requete";
    return mysqli_query($this->linkBrioche, $requete);
  }
  
  /**
  * compare la valeur d'un champ avec une valeur donne
  * @param mixed item l'identifiant de l'enregistrement
  * @param string cle le nom du champ correspondant  item
  * @param string le nom du champ contenant la valeur  comparer
  * @param mixed valeur la valeur avec laquelle comparer la valeur du champ
  * @param boolean s'agit-il d'un champ date ?
  * @return int -1 si $valeur est infrieur  la valeur du champ, +1 dans le cas contraire, 0 si galit
  */
  function compare($item, $cle, $champ, $valeur, $date = false) {
    $requete = "SELECT $champ FROM $this->table WHERE $cle = $item";
    $result = mysqli_query($this->linkBrioche, $requete);
    $ligne = mysqli_fetch_row($result);
    $valcomp = $ligne[0];
    if ($date == 1) { //date-Fr
      $gda = $this->getDateAm($valeur);
      $valeur = strtotime($this->getDateAm($valeur, 0));
      $valcomp = strtotime($valcomp);
    }
    $diff = $valeur - $valcomp;
    if ($diff < 0) {
      return -1;
    }
    elseif ($diff == 0){
      return 0;
    }
    else {
      return 1;
    }
  }

  /**
  * renvoie le nombre d'enregistrements concerns
  * @return int
  */
  function compte(){
    $tab = $this->executeReq($this->getSql());
    return count($tab);
  }

  /**
  * renvoie la somme des valeurs d'un champ donn ( rserver  un champ calcul)
  * @param string champ le nom du champ
  * @return int
  */
  function sommeTab($champ){
    $tabl = $this->getTableau();
    $cpt = count($tabl);
    $tchamp = array();
    for ($i = 0; $i < $cpt; $i++){
      $tchamp[] = $tabl[$i]->$champ;
    }
    if (count($tchamp)) return array_sum($tchamp);
    return 0;
  }
  
  
  /**
  * renvoie la somme d'un champ de la table (ou jointure)
  * @param string champ le nom du champ
  * @return int
  */
  function somme($champ){
    $requete = "SELECT SUM($champ) FROM $this->table ";
    if ($this->join && $this->on){
      if (is_array($this->join)){
        $cpte = count($this->join);
      }
      else {
        $cpte = 1;
      }
      if ($cpte == 1){
        $requete .= "LEFT JOIN $this->join ON $this->on ";
      }
      else {
        for($i = 0; $i<$cpte; $i++){
          $requete .= "LEFT JOIN " . $this->join[$i] . " ON " . $this->on[$i] . " ";
        }
      }
    }
    $requete .= "WHERE ($this->table.tst > $this->t1) AND ($this->where)";
    //print $requete;
    $result = mysqli_query($this->linkBrioche, $requete);
    if (! $result) {
      return 0;
    }
    $ligne = mysqli_fetch_row($result);
    return $ligne[0];
  }
  
  /**
  * renvoie la moyenne d'un champ de table (ou de jointure)
  * @param string champ le nom du champ
  * @param int decimales le nombre de dcimales
  * @return float
  */
  function moyenne($champ, $decimales = 2){
    $requete = "SELECT AVG($champ) FROM $this->table ";
    if ($this->join && $this->on){
      if (is_array($this->join)){
        $cpte = count($this->join);
      }
      else {
        $cpte = 1;
      }
      if ($cpte == 1){
        $requete .= "LEFT JOIN $this->join ON $this->on ";
      }
      else {
        for($i = 0; $i<$cpte; $i++){
          $requete .= "LEFT JOIN " . $this->join[$i] . " ON " . $this->on[$i] . " ";
        }
      }
    }
    $requete .= "WHERE ($this->table.tst > $this->t1) AND ($this->where)";
    //print $requete;
    $result = mysqli_query($this->linkBrioche, $requete);
    if (! $result) {
      return 0;
    }
    $ligne = mysqli_fetch_row($result);
    return round($ligne[0], $decimales);
  }

  /**
  * renvoie un percentile des valeurs d'un champ de la table (ou jointure)
  * @param string champ le nom du champ
  * @param int pourcent la valeur de percentile (O - 100), par dfaut 50 (mdiane)
  * @return int
  */
  function percentile($champ, $pourcent = 50){
    $cpt = $this->compte();
    $nb = round($cpt * $pourcent / 100, 0);
    $requete = "SELECT $champ FROM $this->table ";
    if ($this->join && $this->on){
      if (is_array($this->join)){
        $cpte = count($this->join);
      }
      else {
        $cpte = 1;
      }
      if ($cpte == 1){
        $requete .= "LEFT JOIN $this->join ON $this->on ";
      }
      else {
        for($i = 0; $i<$cpte; $i++){
          $requete .= "LEFT JOIN " . $this->join[$i] . " ON " . $this->on[$i] . " ";
        }
      }
    }
    $requete .= "WHERE ($this->table.tst > $this->t1) AND ($this->where)";
    $requete .= " ORDER BY $champ LIMIT $nb";
    $result = mysqli_query($this->linkBrioche, $requete);
    if (! $result) {
      return 0;
    }
    $ll[0] = "";
    while($ligne = mysqli_fetch_row($result)){
      $ll = $ligne;
    }
    return $ll[0];
  }
}

?>
