<?php
// $Id: grilleOccupation.inc,v 1.1.1.1 2010-10-11 22:25:47 ob Exp $
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
/**
* Grille d'occupation des lits
* @package donnes
*/
/**
* Grille d'occupation des lits
* Pour une priode donne, construit un planning abstrait (indpendant de tout affichage)
* contenant des lits bien dtermins et offre, pour eux, des fonctions de test et d'information. Permet aussi
* d'agir sur le planning, par exemple en attribuant un lit pour la priode considre.
* @package donnes
* @author ob thn cjd jdu
*/
class grilleOccupation extends connexionBrioche {
  /**
  * la date de dbut (timestamp unix)
  * @var int
  */
  var $deb;
  
  /**
  * la date de fin (timestamp unix)
  * si ni la date de fin ni le nombre de jours ne sont fournis, la grille se limite  un seul jour 
  * correspondant  deb. Si nbJours est fourni, fin est ignor
  * @var int
  */
  var $fin;
  
  /**
  * le nombre de jours de la priode couverte par la grille
  * Prime sur la variable fin
  * @var int
  */
  var $nbJours;
  
  /**
  * un objet calendrier (permettant d'accder  des vnements)
  * @var object calendrier
  */
  var $mcal;
  
  /**
  * le tableau des lits concerns par la grille courante. Chaque lment du tableau contient
  * lui-mme un tableau de quatre lments correspondant respectivement au numro d'tablissement,
  * au numro de chambre, au numro de lit et  une description libre
  * @var array
  */
  var $tablits;
  
  /**
  * le nombre de lits concerns par la grille courante
  * @var int
  */
  var $cpteLits;
  
  /**
  * tableau contenant, pour chacun des jours de la priode considre, un objet CJour
  * @var array
  */
  var $tabCJours;
  
  /**
  * le numro de l'tablissement concern par la grille courante
  * si non dfini, tous les tablissements
  * @var int
  */
  var $etabl;
  
  /**
  * le numro de la chambre (dans l'tablissement concern)
  * si non dfini, toutes les chambres
  * @var int
  */
  var $chambre;
  
  /**
  * le numro du lit (dans la chambre et l'tablissement concerns)
  * si non dfini, tous les lits
  * @var int
  */
  var $lit;
  
  /**
  * le tableau associatif des champs de formulaire posts
  * Notamment 'etabl', 'chambre', 'lit', 'urgent', 'sortie_confirmee', 'commentaire'
  * @var array
  */
  var $donnees;
  
  
  /**
  * le sjour envisag est-il urgent ? oui : 1,  non : 0
  * @var int
  */
  var $urgent;
  
  /**
  * la sortie de l'usager au terme du sjour est-elle confirme ? oui : 1, non : 0
  * @var int
  */
  var $sortie_confirmee;
  
  /**
  * commentaire libre li  un sjour
  * @var string
  */
  var $commentaire;
  
  /**
  * sexe (pour un sjour bloqu, c'est--dire, sans attribution  un usager dtermin) : F ou M
  * @var string
  */
  var $sexe;
  
  /**
  * Spcifier si un sjour doit tre pris en compte dans les statistiques
  * @var boolean
  */
  var $horsStats;
  
  /**
  * La date  laquelle le court-sjour se transforme en long sjour
  * @var date
  */
  var $datedecision;
  
  /**
  * tableau associatif dont les cls correspondent  des noms de champs de la base de donnes
  * et servant  une requte mise--jour
  * @var array
  */
  var $tabchamps;

  /**
  * @param int deb cf. variable de classe
  * @param int nbjours cf. variable de classe
  * @param string sessionid cf. variable de classe
  * @param int fin cf. variable de classe
  * @param array donnees cf. variable de classe
  * @param array autspec cf. variable de classe
  */
  function grilleOccupation($deb, $nbJours=0, $sessionid, $fin=0, $donnees=0, $autspec=0) {
    if (! $this->connexionBrioche($sessionid, $autspec)) return;
    $this->deb = $deb;
    $this->fin = $fin;
    if (! $nbJours){
      if (! $fin) {
        $nbJours = 1;
      }
      else {
        $nbJours = round(($fin - $deb)/SEC_PAR_JOUR) + 1;
      }
    }
    $this->nbJours = $nbJours;
    $this->donnees = $donnees;
    if ($this->donnees){
      $this->etabl = $this->donnees['etabl'];
      $this->chambre = $this->donnees['chambre'];
      $this->lit = $this->donnees['lit'];
      $this->urgent = $this->donnees['urgent'];
      $this->sortie_confirmee = $this->donnees['sortie_confirmee'];
      $this->commentaire = $this->donnees['commentaire'];
      $this->sexe = $this->donnees['sexe'];
      $this->horsStats = $this->donnees['horsStats'];
      $this->datedecision = $this->donnees['datedecision'];
    }
    $ini = new briocheIni();
    $this->mcal = new calendrier($sessionid, $this->autspec);
    if ($deb && $nbJours) {
      $premierJour = new CJour(date("Y", $deb), date("m", $deb), date("d", $deb));
      $jour = $premierJour;
      for ($i = 0; $i < $this->nbJours; $i++){
        $this->tabCJours[] = $jour;
        $jour = $this->jourSuivant($jour);
      }
      $this->addLits();
      $this->addEvenements();
    }
  }

  /**
  * attribution d'un lit
  * @param int titre l'identifiant d'un vnement si dj connu (modification d'un sjour existant)
  * @param int le numro de l'annonce de lit si dj connu (modification d'un sjour existant)
  * @return int le numro de l'vnement correspondant au sjour
  */
  function attribueLit($titre = 0, $nolit = 0){
    $mcal = &$this->mcal;
    $vx = new verrouCal($this->idsession, $this->autspec);
    if(! $vx->ferme()) return;
    if (! $this->donnees){
      print "Donnes lit non spcifies";
    }
    $donnees = $this->donnees;
    $id = 0;
    
    if ($this->checkDisponibilite($titre)){
      if($titre){
        $ev = $mcal->fetchEvent($titre);  
      }
      else {
        $ev = $mcal->eventInit();
      }
      $id = $ev->noev;
      $this->tabchamps['debut'] = date("j", $this->deb) . "." . date("m", $this->deb) . "." . date("Y", $this->deb);
      $this->tabchamps['fin'] = date("j", $this->fin) . "." . date("m", $this->fin) . "." . date("Y", $this->fin);
      $this->tabchamps['nodoss'] = $donnees['nodoss'];
      $this->tabchamps['etablissement'] = $this->etabl;
      $this->tabchamps['chambre'] = $this->chambre;
      $this->tabchamps['lit'] = $this->lit;
      $this->tabchamps['nolit'] = "NULL";
      $this->tabchamps['urgent'] = $this->urgent;
      $this->tabchamps['sortie_confirmee'] = $this->sortie_confirmee;
      $this->tabchamps['commentaire'] = $this->commentaire;
      $this->tabchamps['sexe'] = $this->sexe;
      $this->tabchamps['horsStats'] = $this->horsStats;
      $this->tabchamps['datedecision'] = $this->datedecision;
        
      $ini = new briocheIni();
      
      // AU stade actuel, le proprio est toujours le brio !
      if ($this->groupe == $ini->groupeCBRIO || $this->groupe == $ini->groupeDEVBRI || $this->groupe == $ini->groupeRESPBRI ) {
        $proprio = "BRIO";
      }
      else {
        $proprio = "EMS";
      }
      $this->tabchamps['proprio'] = $proprio;
      $retour = 0;
      
      //Gnration ou modification de l'annonce de lit


      if ($donnees['nodoss'] != -999){
        $perso = new ensTableBrioche("dossiers", "nodoss = $donnees[nodoss]", "usagers.sexe", 0,
                                     $this->idsession, "usagers", "dossiers.nous = usagers.nous");
        $tperso = $perso->getTableau();
        $tperso = $tperso[0];
        $litsch = etablissementBrioche::nbLits($donnees['etabl'], $donnees['chambre'], $this->idsession);
        $litsch = ($litsch > 3 ? 3 : $litsch);
        $mission = etablissementBrioche::mission($donnees['etabl'], $this->idsession);
        $typesej = 5;
        $annl = new ensTableBrioche("annoncelit", 1, "", 0, $this->idsession);
        if (! $nolit){
          $nolit = $annl->creeNouveau("nolit");
        }
        $this->tabchamps['nolit'] = $nolit;
        
        $tabann['etabl'] = $donnees['etabl'];
        $tabann['litdepuis'] = $donnees['depuis'];
        $tabann['stlit'] = $typesej;
        $tabann['courtau'] = $donnees['jusqua'];
        $tabann['sexlit'] = ($tperso->sexe == "M" ? 1 : 2);
        $tabann['chambre'] = $litsch;
        $tabann['datann'] = $annl->aujourdhui();
        $tabann['nomann'] = "Brioche";
        $tabann['mission'] = $mission;
        $annl = new updateTableBrioche("annoncelit", $tabann, $nolit, "nolit", $this->idsession);
        if ($annl->update()) {
          $retour++;
        }
        else {
          //Il faudra ajouter un rollback sur le calendrier
          return 0;
        }
      }

      //Ecriture dans attributionlitz2
      
      if ($donnees['nodoss'] != -999){
        $tabann['dateres'] = $annl->aujourdhui();
        $tabann['dateplac'] = $donnees['depuis'];
        $tabann['nolit'] = $nolit;
        $tabann['typesej'] = $typesej;
        $abann['mission'] = $mission;
        $annl = new updateTableBrioche("attributionlit", $tabann, $donnees['nodoss'], "nodoss", $this->idsession);
        if ($annl->update()){
          $retour++;
        }
        else {
          //Il faudra ajouter un rollback sur annoncelit et le calendrier
          print "ERREUR";
          return 0;
        }
      }
      //Ecriture dans issue
      if ($donnees['nodoss'] != -999){
        $tabann['dateissue'] = $donnees['jusqua'];
        $tabann['issue'] = 11;
        $annl = new updateTableBrioche("issue", $tabann, $donnees['nodoss'], "nodoss", $this->idsession);
        if ($annl->update()){
          $retour++;
        }
        else {
          //rollback  ajouter
          print "ERREUR2";
          return 0;
        }
      }
      //Ecriture dans demandeor
      if ($donnees['nodoss'] != -999){
        $tabann['litdepuis'] = $donnees['depuis'];
        $tabann['litjusqua'] = $donnees['jusqua'];
        $tabann['typesej'] = 5;
        $annl = new updateTableBrioche("demandeor", $tabann, $donnees['nodoss'], "nodoss", $this->idsession);
        if ($annl->update()){
          $retour++;
        }
        else{
          print "ERREUR3";
        }
      }
      //Ecriture dans dossier
      if ($donnees['nodoss'] != -999){
        if(strtotime(connexionBrioche::getDateAm($donnees['jusqua'], 0)) < time()){  // La date d'issue est passe -> archiver dossier (pour les CS planning, date d'issue = FIN du sjour)
          $tabann['dateclo'] = $donnees['jusqua'];
        }
        else {
          $tabann['dateclo'] = "";
        }    
        $annl = new updateTableBrioche("dossiers", $tabann, $donnees['nodoss'], "nodoss", $this->idsession);
        if ($annl->update()){
          $retour++;
        }
        else { // Pas de rollback ici : si le reste s'est correctement crit, faitMenage corrigera le problme
          print "ERREUR3";
          return 0;
        }
      }
      if($mcal->setChamps($id, $this->tabchamps)){
        $retour++;
      }
      else {
        return 0;
      }
    }
    $vx->rouvre();    
    //Passerelle sulid
    $pass = new passSulid($this->idsession, $this->autspec);
    return $id;
  }

  /**
  * Vrification de la disponibilit
  * @param int titre un numro d'vnement : 
  * si dfini, exclut du contrle de disponibilit de sjour portant cet identifiant) 
  * @return boolean le lit est-il libre sur la priode considre ?
  */
  function checkDisponibilite($titre){
    // On parcourt les jours de la grille d'occupation et si dans le calendrier les jours sont compltement occups par 
    // un usager, on dit  l'utilisateur que le lit n'est pas libre.
    $libre = true;
    for ($i = 0; $i < $this->nbJours; $i++) {
      $jour = &$this->tabCJours[$i];

      $jour_courant_occupation = $jour->isOccupe($this->etabl, $this->chambre, $this->lit, $titre);
      if ($i == 0) {
        // premier jour
        $jour_apres = &$this->tabCJours[$i+1];
        if (! isset($jour_apres)) {
          $libre = true;
        }
        elseif ($jour_courant_occupation && $jour_apres->isOccupe($this->etabl, $this->chambre, $this->lit, $titre)) {
          $libre = false;
        }
      } elseif ($i == $this->nbJours-1) {
        // dernier jour
        $jour_avant = &$this->tabCJours[$i-1];
        if ($jour_courant_occupation && $jour_avant->isOccupe($this->etabl, $this->chambre, $this->lit, $titre)) {
          $libre = false;
        }
      } 
      else {
        if ($jour_courant_occupation){
        $libre = false;
        }
      }  
      if (!$libre) {
        $al = new Alerte("Ce lit n\'est pas libre  la priode indique");
        $al->imprime();
        return false;
      }
    }
    return true;
  }

  /**
  * renvoie un objet CJour correspondant au jour qui suit celui pass en paramtre
  * @param object auj un objet CJour
  * @return object objet CJour du lendemain
  */
  function jourSuivant(&$auj){
    $tst = mktime(0, 0, 0, $auj->mois, $auj->jour, $auj->annee) + SEC_PAR_JOUR + 3700;
    $suiv = new CJour(date("Y", $tst), date("m", $tst), date("d", $tst));
    return $suiv;
  }

  /**
  * construction du tableau des lits concerns
  * ce tableau est crit dans la proprit tablits
  */
  function addLits(){
    $whereetabl = ($this->etabl ? "etabl = $this->etabl " : "1 ");
    $wherechambre = ($this->chambre ? "AND chambre = $this->chambre " : "AND 1 ");
    $wherelit = ($this->lit ? "AND lit = $this->lit" : "AND 1 ");
    $debut = date('Ymd',$this->deb);
    $fin = date('Ymd',$this->deb+$this->nbJours*86400);
    $wheredate = "AND (ISNULL(debut) OR debut < " . $fin . ") AND (ISNULL(fin) OR fin > " . $debut . ")";
    $tlits = new EnsTableBrioche("litscourtsejour", ($whereetabl . $wherechambre . $wherelit . $wheredate),
                                 "etabl, chambre, lit, descr, debut, fin", "etabl, chambre, lit", $this->idsession, 0, 0, 
                                 false, $this->autspec);
    $lits = $tlits->getTableau();
    $this->cpteLits = count($lits);
    for ($i = 0; $i < $this->cpteLits; $i++){
      $this->tablits[] = array ($lits[$i]->etabl, $lits[$i]->chambre, $lits[$i]->lit, $lits[$i]->descr, $lits[$i]->debut, $lits[$i]->fin);
    }
  }

  /**
  * nombre de lit d'un tablissement donn
  * @param int etabl le numro d'tablissement
  * @return int le nombre de lits
  */
  function getNbLits($etabl){
    $nblits = 0;
    for ($i = 0; $i < $this->cpteLits; $i++) {
      if ($this->tablits[$i][0] == $etabl) {
        $nblits++;
      }
    }
    return $nblits;
  }


  /**
  * nombre de chambres d'un tablissement donn
  * @param int etabl le numro d'tablissement
  * @return int le nombre de chambres
  */
  function getNbChambres($etabl){
    $nbch = 0;
    $chprec = array();
    for ($i = 0; $i < $this->cpteLits; $i++){
      if($this->tablits[$i][0] == $etabl){
        if ($this->tablits[$i][1] != $chprec){
           $chprec = $this->tablits[$i][1];
           $nbch++;
        }
      }
    }
    return $nbch;
  }

  /**
  * extrait le tableau des lits d'un tablissement  partir de la probprit tablits
  * @param int etabl le numro d'tablissement
  * @return array
  */
  function getTabLits($etabl){
    for ($i = 0; $i < $this->cpteLits; $i++){
      if($this->tablits[$i][0] == $etabl){
        $tl[] = array ($this->tablits[$i][1], $this->tablits[$i][2], $this->tablits[$i][3], $this->tablits[$i][4], $this->tablits[$i][5] );
      }
    }
    return $tl;
  }


  /**
  * rcupration de tous les vnements depuis le dbut de l'anne prcdente
  * jusqu' la fin de l'anne suivante (date de rfrence : deb)
  */
  function addEvenements(){
    $mcal = &$this->mcal;
    $dte = getdate($this->deb);
    $events = $mcal->listEvents(($dte['year'] - 1), 1, 1,
                               ($dte['year'] + 1), 12, 31);
    $cpt = count($events);
    for ($j = 0; $j < $cpt; $j++){
      $ev = $events[$j];
      $deb = $this->dateFr2tst($ev->debut);
      $fin = $this->dateFr2tst($ev->fin);
      $duree = round(($fin - $deb)/SEC_PAR_JOUR);
      for ($i = 0; $i < $this->nbJours; $i++){
        $jour = &$this->tabCJours[$i];
        if ($jour->tst >= $deb && $jour->tst <= $fin){
          $nojour = round(($jour->tst - $deb)/SEC_PAR_JOUR); //nb de jours avant le dbut du planning
          $jour->setOccupe($ev->etablissement, $ev->chambre, $ev->lit,
                           $ev->nodoss, $duree, $nojour, $deb, $fin, $ev->proprio, $ev->noev, $ev->urgent,
                           $ev->sortie_confirmee, $ev->sexe);
        }
      }
    }
  }
  
  /**
  * fusion d'ventuels sjours fragments dont on aurait replac les fragments dans le mme lit
  */
  function fusionneFragments(){
    $mcal = &$this->mcal;
    while($afusionner = $mcal->getAFusionner()){
      $ev = array();
      // g pour "garder", s pour "supprimer"
      $tit['g'] = $afusionner->noev;
      $tit['s'] = $afusionner->noev1;
      $ev['g'] = $mcal->fetchEvent($tit['g']);
      $ev['s'] = $mcal->fetchEvent($tit['s']);
      //la plus anciennes des dates de dbut
      $datedebut = min($this->dateFr2Tst($ev['g']->debut), $this->dateFr2Tst($ev['s']->debut));
      $datefin = max($this->dateFr2Tst($ev['g']->fin), $this->dateFr2Tst($ev['s']->fin));
      //Destruction du sjour-vnement  supprimer
      $ses = new sejourEvenement($tit['s'], $this->idsession);
      $ses->detruitEvenement(false); // Destruction du sjour-vnement, sans clture du dossier
      //Attribution du lit gard, avec modification des dates
      $nolit = $ev['g']->nolit;
      $fu = new grilleOccupation($datedebut, 0, $this->idsession, $datefin, $this->donnees, $this->autspec);
      $fu->attribueLit($tit['g'], $nolit);
      
    }
    $mcal->nettoieEnchainements();
  }
}



?>
