<?php
// $Id: sejourEvenement.inc,v 1.1.1.1 2010-10-11 22:25:49 ob Exp $
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
/**
* reprsentation abstraite d'un sjour
* @package donnes
*/
/**
* reprsentation abstraite d'un sjour
* le sjour est envisag ici comme un vnement de calendrier, associ  un lit et  un dossier d'usager
* @package donnes
* @author ob tnh cjd jdu
*/
class sejourEvenement extends connexionBrioche{
  /**
  * le numro de l'vnement (champ noev de la table "calendrier")
  * @var int
  */
  var $numero;
  
  /**
  * l'objet calendrier utilis
  * @var object calendrier
  */
  var $mcal;
  
  /**
  * l'objet vnement correspondant associ  ce sjour.
  * En fait un objet correspondant  un enregistrement de la table calendrier
  * @var object 
  */
  var $ev;
   
  /**
  * @param int numero cf. variable de classe
  * @param string sessiondi l'identifiant de session
  * @param array autspec le tableau des autorisations spcifiques
  */
  function sejourEvenement($numero, $sessionid, $autspec = ""){
    if (! $this->connexionBrioche($sessionid, $autspec)) return;
    $this->numero = $numero;
    $ini = new BriocheIni();
    $this->mcal = new calendrier($sessionid, $this->autspec);
    $this->ev = $this->mcal->fetchEvent($this->numero);
  }

  /**
  * renvoie l'objet vnement associ  ce sjour. Une proprit par champ de la table calendrier
  * @return object
  */
  function getEvenement(){
    return $this->ev;
  }
  
  /**
  * existe-t-il, pour le mme dossier usager, un sjour suivant dans un autre lit ?
  * @return int le nombre de sjours correspondants
  */
  function hasSuivant(){
    $et = new ensTableBrioche("calendrier",
                              "calendrier.noev = $this->numero and calendrier1.noev is not null and calendrier1.actif is not null", 
                              "calendrier1.noev", 0, $this->idsession,
                              "calendrier as calendrier1", "calendrier.noev = calendrier1.csprec");
    return $et->compte();                            
  }
  
  /**
  * existe-t-il, pour le mme dossier,  un sjour prcdent dans un autre lit ?
  * @return int le nombre de sjours correspondants
  */
  function hasPrecedent(){
    $et = new ensTableBrioche("calendrier", "calendrier.noev = $this->numero and calendrier.csprec is not null",
                              "calendrier.noev", 0, $this->idsession);
    return $et->compte();                          
  }
  
  /**
  * destruction d'un sjour-vnement, c'est--dire
  * <pre>
  * - suppression de l'vnement calendrier
  * - libration du lit
  * - retrait de l'annonce de lit
  * - retrait de la demande d'orientation et archivage du dossier
  * </pre>
  */
  /**
  * @param boolean clot
  * le dossier doit-il tre archiv ?
  * (Dans le cas o le sjour supprim n'est qu'un des vnements correspondant  un sjour fractionn
  * le dossier devra rester ouvert et clot sera donc paramtr sur false)
  */
  function detruitEvenement($clot = true){
    $nodoss = $this->ev->nodoss;
    $vx = new verrouCal($this->idsession, $this->autspec);
    if (! $vx->ferme()) return;
    $et = new ensTableBrioche("calendrier",
                              "nodoss = " . $this->ev->nodoss . " AND actif = 1",
                              "noev", 0, $this->idsession);
    $nbdoss = $et->compte();
    if ($nodoss != -999){
      $ea = new enrAnnonce($this->ev->nolit, $this->idsession);
      //libration du lit
      if(! $ea->libere()){
        print "PROBLEME 1";
      }
      else {
        //Retrait l'annonce de lit
        if(! $ea->retire()){
          print "PROBLEME 2";
        }
        else {  
          //Retrait de la demande et cloture du dossier
          if ($clot && $nbdoss == 1) {
            $enr = new enrBrioche($nodoss, $this->idsession);
            $enr->clotSurRetraitCS();
          }
          elseif ($nbdoss > 1){
            $this->majDates($this->getTermes(false));      
          }
        }    
      }  
      
      
    }  
    $rt = $this->mcal->deleteEvent($this->ev->noev);
    $this->mcal->nettoieEnchainements();
    $vx->rouvre();
    return $rt;
  }
  
  /**
  * renvoie la date du dbut de l'vnement
  * @return string (date au format francophone)
  */
  function getDebut(){
    return $this->ev->debut;    
  }
  
  /**
  * renvoie la date de fin de l'vnement
  * @return string (date au format francophone)
  */
  function getFin(){
    return $this->ev->fin;
  }

  /**
  * le sjour est-il urgent ?
  * @return boolean
  */
  function getUrgence() {
    return $this->ev->urgent;
  }
  
  /**
  * la sortie est-elle confirme ?
  * @return boolean
  */
  function getSortieConfirmee() {
    return $this->ev->sortie_confirmee;
  }
  
  /**
  * pour quel sexe le sjour est-il bloqu ?
  * @return string "M" | "F"
  */ 
  function getSexe() {
    return $this->ev->sexe;
  }
  
  /**
  * renvoie le commentaire associ  ce sjour
  * @return string
  */
  function getCommentaire() {
    return $this->ev->commentaire;
  }
  
  /**
  * renvoie le commentaire associ  ce sjour
  * @return string
  */
  function getHorsStats() {
    return $this->ev->horsStats;
  }
  
  /**
  * renvoie le commentaire associ  ce sjour
  * @return string
  */
  function getDateDecision() {
    return $this->ev->datedecision;
  }

  /**
  * renvoie le numro de l'vnement calendrier associ (champ noev de la table calendrier)
  * @return int
  */
  function getTitre(){
    return $this->ev->noev;
  }
  
  /** 
  * renvoie les caractristiques du lit
  * @return array 
  * <pre>
  * (0 => numro d'tablissement, 1 => numro de chambre, 2 => numro du lit, 3 => nom tablissement,
  * 4 => numro de l'annonce de lit) 
  * </pre>
  */
  function getLit(){
    $lit[0] = $this->ev->etablissement;
    $lit[1] = $this->ev->chambre;
    $lit[2] = $this->ev->lit;
    $et = new ensTableBrioche("etablissements", "noet = $lit[0]", "nomet", "", $this->idsession,
                              0, 0, false, $this->autspec);
    $ett = $et->getTableau();
    $lit[3] = $ett[0]->nomet;
    $lit[4] = $this->ev->nolit;
    return $lit;
  }
  
  /**
  * renvoie les caractristiques du dossier
  * @return array
  * <pre>
  * (0 => numro du dossier, 1 => nom prnom ou "Sjour bloqu"
  * </pre>
  */
  function getDossier(){
    $dossier[0] = $this->ev->nodoss;
    if ($dossier[0] == -999) {
      $dossier[0] = "";
      $dossier[1] = "Sjour bloqu - pas de dossier attribu";
    }
    else {
      $enr = new enrBrioche($dossier[0], $this->idsession, $this->autspec);
      $pers = $enr->getEnr("usagers");
      $dossier[1] = "$pers->nom $pers->prenom";
      $dossier[2] = $pers->nous;
    }
    return $dossier; 
  }

  /**
  * renvoie les initiales de la personne suivant le dossier
  * @return array
  * <pre>
  * (0 => numro du dossier, 1 => initiales ou "Sjour bloqu"
  * </pre>
  */
  function get_suivi_par() {
  	$dossier[0] = $this->ev->nodoss;
  	if ($dossier[0] == -999) {
      $dossier[0] = "";
      $dossier[1] = "Sjour bloqu - pas de dossier attribu";
    }
    else {
      $enr = new enrBrioche($dossier[0], $this->idsession, $this->autspec);
      $suivi_par = $enr->getEnr("dossiers");
      $dossier[1] = $suivi_par->suivpar;
    }
    return $dossier;
  }
  
  /**
  * mise  jour de toutes les dates, aprs modification de celles du sjour
  * @param array termes un tableau avec le dbut et la fin du sjour
  */
  function majDates($termes){
    //Table attributionlit
    $tabchamps = array();
    $tabchamps['dateplac'] = $termes['debut'];
    $ut = new updateTableBrioche("attributionlit", $tabchamps, $this->ev->nodoss, "nodoss", $this->idsession);
    $ut->update();
    //Table issue
    $tabchamps = array();
    $tabchamps['dateissue'] = $termes['fin'];
    $ut = new updateTableBrioche("issue", $tabchamps, $this->ev->nodoss, "nodoss", $this->idsession);
    $ut->update();
    //Table demandeor
    $tabchamps = array();
    $tabchamps['litdepuis'] = $termes['debut'];
    $tabchamps['litjusqua'] = $termes['fin'];
    $ut = new updateTableBrioche("demandeor", $tabchamps, $this->ev->nodoss, "nodoss", $this->idsession);
    $ut->update();
    //Table dossiers
    $tabchamps = array();
    if(strtotime($this->getDateAm($termes['fin'], 0)) < time()){
      // La date d'issue est passe -> archiver dossier (pour les CS planning, date d'issue = FIN du sjour)
      $tabchamps['dateclo'] = $termes['fin'];
    }
    else {
      $tabchamps['dateclo'] = "";
    } 
    $ut = new updateTableBrioche("dossiers", $tabchamps, $this->ev->nodoss, "nodoss", $this->idsession);
    $ut->update();
  }
  
  /**
  * renvoie les dates extrmes des fragments de sjour attribus  un dossier, avec ou sans ce sjour-ci ($this)
  * @param boolean mecompter faut-il compter le sjour courant ?
  * @return array ('debut' => la date de dbut, 'fin => la date de fin) dates au format francophone
  */
  function getTermes($mecompter = true){
    $et = new ensTableBrioche("calendrier",
                              "actif = 1 AND nodoss = " . $this->ev->nodoss .
                              ($mecompter ? "" : " AND noev <> $this->numero"),
                              "debut", "debut", $this->idsession);
    $tet = $et->getTableau();
    $retour = array();
    $retour['debut'] = $tet[0]->debut;
    $et->setChamps("fin");
    $et->setOrder("fin DESC");
    $tet = $et->getTableau();
    $retour['fin'] = $tet[0]->fin;
    return $retour;                          
  }

}
?>
