<?php
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
/**
* Gestion de tables provisoires
* @package accessoires
*/
/**
* Gestion de tables provisoires
* Cration, maniement et destruction de tables provisoires
* @package accessoires
* @author ob
*/
class tableProv extends connexionBrioche{
  /**
  * le nom de la table
  * @var string
  */
  var $nomTable;
  
  /**
  * le tableau des champs de la table
  * @var array
  */
  var $champs;
  
  /**
  * le nombre de champs
  * @var int
  */
  var $nbchamps;
  
  /**
  * le prfixe de table provisoire
  * @var string
  */
  var $prefixe;
  
  /**
  * @param string un identifiant pour la table  crer
  * @param string l'identifiant de session
  * @param string le nom de la table
  */
  function __construct($id, $sessionid, $nom = false){
    if (! parent::__construct($sessionid)) return false;
    if (! $nom){
      $this->prefixe = "_prov";
      $this->nomTable = $this->prefixe . $id . "_" . $this->idsession;
    }
    else { // On dsire nommer la table provisoire (ou technique), donc elle ne sera pas dtruite automatiquement
      $this->prefixe = "_";
      $this->nomTable = $this->prefixe . $nom; // Par contre un _ est ajout en dbut de nom
    }
  
  }

  /**
  * dfinition d'un champ
  * @param string nom le nom du champ
  * @param string type le type du champ
  * @param int taille la taille du champ
  * @param string attributs un attribut supplmentaire
  * @param boolean null la valeur NULL est-elle autorise ?
  * @param string defaut la valeur par defaut
  * @param boolean autoincr le champ est-il auto-incrmental ?
  * @param boolean pri le champ est-il la cl primaire ?
  */
  function setChamp($nom, $type, $taille = "", $attributs = "", $null = true,
                    $defaut = "", $autoincr = false, $pri = ""){
    $i = (isset($this->champs) ? count($this->champs) : 0);
    $this->champs[$i]['nom'] = $nom;
    $this->champs[$i]['type'] = $type;
    $this->champs[$i]['taille'] = $taille;
    $this->champs[$i]['attributs'] = $attributs;
    $this->champs[$i]['null'] = $null;
    $this->champs[$i]['defaut'] = $defaut;
    $this->champs[$i]['autoincr'] = $autoincr;
    $this->champs[$i]['pri'] = $pri;                  	
  }

  /**
  * cration effective de la table en fonction des champs dfinis
  */
  function create(){
    $req = "CREATE TABLE " . $this->nomTable . " (";
    $cpt = (isset($this->champs) ? count($this->champs) : 0);
    for ($i = 0; $i < $cpt; $i++){
      $guillemets = true;
      if(preg_match("/timestamp/i", $this->champs[$i]['type'])) $guillemets = false;
      $req .= $this->champs[$i]['nom'] . " " . $this->champs[$i]['type'] .
              ($this->champs[$i]['taille'] ? "(" . $this->champs[$i]['taille'] . ")" : "") .
              ($this->champs[$i]['attributs'] ? " " . $this->champs[$i]['attributs'] . " " : "") .
              ($this->champs[$i]['null'] ? " NULL " : " NOT NULL ") .
              (strlen($this->champs[$i]['defaut']) > 0 ? " DEFAULT " . ($guillemets ? "'" : "") .
              $this->champs[$i]['defaut'] . ($guillemets ? "'" : "") . " " : "") .
              ($this->champs[$i]['autoincr'] ? " AUTO_INCREMENT " : "") .
              ($this->champs[$i]['pri'] ? " PRIMARY KEY " : "");
              $req = trim($req);
              $req .= ", ";
    }	   	
    $req = substr($req, 0, -2) . " ) ENGINE MyIsam;";
    //print "{" . $req . "}";
    $drop = "DROP TABLE IF EXISTS $this->nomTable";
    $result = mysqli_query($this->linkBrioche, $drop);
    $result = mysqli_query($this->linkBrioche, $req);
    
  }	
  
  /**
  * Une table  ce nom existe-t-elle dans la base en cours ?
  * @return boolean
  */
  function exists() {
    $req = "SHOW TABLES";
    $result = mysqli_query($this->linkBrioche, $req);
    while ($ligne = mysqli_fetch_row($result)){
      $tab[] = $ligne[0];
    }
    return in_array($this->nomTable, $tab);
  }
  
  
  /**
  * renvoie le nom de la table
  * @return string
  */
  function getNom(){
    return $this->nomTable;
  }	
  
  /**
  * renvoie un fichier CSV contenant les donnes de la table
  * @param string order l'ordre de tri
  * @return string
  */
  function getCSV($order){
    //Nom des champs
    $contenu = "";
    $requete = "SHOW FIELDS FROM $this->nomTable";
    $result = mysqli_query($this->linkBrioche, $requete);
    $nc = 0;
    while($ligne = mysqli_fetch_row($result)){
      if ($ligne[0] != "tst"){
        $champs[] = $ligne[0];
        $contenu .= "\"$ligne[0]\"\t";
        $nc++; 
      }
    }
    $this->nbchamps = $nc;
    $contenu = substr($contenu, 0, -1);
    $contenu .= "\n";
    //Variables
    $et = new ensTableBrioche("$this->nomTable", 1, "*", $order, $this->idsession);
    $tet = $et->getTableau();
    $cpt = count($tet);
    for($i = 0; $i < $cpt; $i++){
      for($j = 0; $j < $this->nbchamps; $j++){
        $nomchamp = $champs[$j];
        $contenu .= "\"" . addslashes($tet[$i]->$nomchamp) . "\"\t";	
      }
      $contenu = substr($contenu, 0, -1);
      $contenu .= "\n";
    }	 
    return $contenu;		
  }	
  
  /**
  * destruction de toutes les tables provisoires cres par des sessions chues
  * @param string id identifiant  prciser si l'on veut limiter la destruction  un identifiant
  */
  function nettoie($id = ""){
    $requete = "SHOW tables";
    $result = mysqli_query($this->linkBrioche, $requete);
    $tablesprov = array();
    while($ligne = mysqli_fetch_row($result)){
      if (preg_match("/^_prov$id/i", $ligne[0])){
        $tablesprov[] = $ligne[0];  
      }
    }  
    $cpt = count($tablesprov);
    for ($i = 0; $i < $cpt; $i++){
       $sessionid = substr($tablesprov[$i], -32);
       if(! Session::estActive($sessionid)){
         $requete2 = "DROP TABLE IF EXISTS " . $tablesprov[$i];
         $result2 = mysqli_query($this->linkBrioche, $requete2);  
       }
    }	
  }	

}



?>
