<?php
// $Id: CJour.inc,v 1.2 2017/07/06 22:23:37 ob Exp $
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
/**
* Objet "Jour"
* @package outils
*/
/**
* Objet "Jour", utilisable dans une grille
*
* L'objet jour peut tre utilis comme proprit d'un objet grilleOccupation.
* Il contient un certain nombre de proprits ayant trait d'une part aux particularits du calendrier
* (par exemple jours fris), et permet de stocker des donnes d'occupation (pour des lits d'EMS)
* Ne requiert plus que php soit compil avec l'extension mcal
* <pre>
* Exemple :
* $premierJour = new CJour(date("Y", $deb), date("m", $deb), date("d", $deb));
* if $premierJour->isFerie();
* print "C'est un jour fri !";
* </pre>
* @author ob
* @package outils
*/
class CJour {
  /**
  * l'anne (4 chiffres)
  * @var int
  */
  var $annee;
  /**
  * le mois (1-12)
  * @var int
  */
  var $mois;
  /**
  * le quantieme (1-31)
  * @var int
  */
  var $jour;
  /**
  * le timestamp (unix) correspondant au jour reprsent
  * @var int
  */
  var $tst;
  /**
  * la couleur de fond (lorsque reprsent dans un planning
  * @var string
  */
  var $couleur;
  /**
  * Un tableau contenant, le cas chant, le numro du dossier pour un lit.
  *
  * Cette proprit est un tableau index $occupation[$etablissement][$chambre][$lit][$index]. Si le lit
  * concern est occup  la date reprsente, il contient le numro du dossier. Le dernier niveau du
  * tableau, l'index 0 ou 1, permet de grer les sjours accols (la situation o un nouvel usager est admis
  * le jour-mme de la sortie du prcdent).
  * @var array
  */
  var $occupation;
  /**
  * Un tableau contenant la dure du sjour (si le lit est occup)
  *
  * Cette proprit est un tableau index $duree[$etablissement][$chambre][$lit][$index]. Si le lit
  * concern est occup  la date reprsente, elle contient la dure du sjour. En cas de sjours accols,
  * les index [0] et [1] contiennent les deux dures
  * @var array
  */
  var $duree;
  /**
  * Le numro d'ordre du jour, par rapport au dbut du sjour
  *
  * Cette proprit est un tableau index  $nojour[$etablissement][$chambre][$lit][$index]. Si le lit
  * concern est occup  la date reprsente, elle contient le numro d'ordre du jour (0 pour le premier
  * jour du sjour)
  * @var array
  */
  var $nojour;
  /**
  * Le dbut du sjour (timestamp)
  *
  * Cette proprit est un tableau index  $debut[$etablissement][$chambre][$lit][$index]. Si le lit
  * concern est occup  la date reprsente, elle contient le timestamp du dbut du sjour.
  * @var array
  */
  var $debut;
  /**
  * La fin du sjour (timestamp)
  *  cf. debut
  * @var array
  */
  var $fin;
  /**
  * Le "propritaire" du sjour (BRIO ou tablissement)
  * @var string
  */
  var $proprio;
  /**
  * le numro de l'vnement (champ noev de la table calendrier)
  * @var string
  */
  var $title;
  /**
  * Indique si le sjour est urgent ou non
  * @var boolean
  */
  var $urgent;
  /**
  * Indique si la date de sortie du sjour a t confirme ou pas
  * @var boolean
  */
  var $sortie_confirmee;
  /**
  * Indique le sexe (pour les sjours bloqus, sans dossier attribu)
  * @var string
  */
  var $sexe;
  /**
  * @param int annee, cf. variable de classe
  * @param int mois, cf. variable de classe
  * @param int jour, cf. variable de classe
  */
  function __construct($annee, $mois, $jour){
    $this->annee = $annee;
    $this->mois = $mois;
    $this->jour = $jour;
    $this->tst = mktime(0, 0, 0, $mois, $jour, $annee);
  }

  /**
  * Le jour de la semaine correspondant au jour reprsent (0 pour dimanche)
  * @access public
  * @return int
  */
  function jourSem(){
    $madate = getdate(mktime(0, 0, 0, $this->mois, $this->jour, $this->annee));
    return $madate['wday'];
  }

  /**
  * Le jour est-il fri ?
  * @access public
  * @return boolean
  */
  function isFerie(){
    $ferie = false;
    if ($this->mois == 1 && ($this->jour == 1 || $this->jour == 2)) $ferie = true; //Nouvel-an et 2 janv.
    $paques = easter_date($this->annee);
    if ($this->tst == $paques) $ferie = true; //Paques
    if ($this->tst == strtotime(date("Y-m-d", $paques + SEC_PAR_JOUR))) $ferie = true; //Lundi de Paques;
    if ($this->tst == strtotime(date("Y-m-d", $paques + SEC_PAR_JOUR * 39))) $ferie = true; //Ascension
    if ($this->tst == strtotime(date("Y-m-d", $paques + SEC_PAR_JOUR * 50))) $ferie = true; //Lundi de pentecte
    if ($this->tst == strtotime(date("Y-m-d", $paques - SEC_PAR_JOUR * 2))) $ferie = true; //Lundi de pentecte
    if ($this->mois == 8 && $this->jour == 1) $ferie = true; //1er Aot
    if ($this->mois == 9) {
      for ($i = 1; $i < 8; $i++) {
        $md = getdate(mktime(0, 0, 0, 9, $i, $this->annee));
        if ($md['wday'] == 0){
          $dimanche = $i;
          break;
        }
      }
      $dimanche = $dimanche + 14;
      if ($this->jour == $dimanche + 1) $ferie = true;
    }
    if ($this->mois == 12 && ($this->jour == 25 || $this->jour == 26)) $ferie = true;
    return $ferie;
  }
  /**
  * Renvoie le nom du mois correspondant au jour reprsent (texte)
  * @access public
  * @return boolean
  */
  function monMois(){
    return $this->texteMois($this->mois);
  }
  /**
  * Dfinit une couleur pour le jour reprsent
  * @access public
  * @return string
  */
  function setCouleur($couleur){
    $this->couleur = $couleur;
  }
  /**
  * Dfinit une occupation pour le jour reprsent
  * @param int etabl le numro de l'tablissement
  * @param int ch le numro de la chambre
  * @param int lit le numro du lit
  * @param int doss le numro du dossier
  * @param int duree la dure du sjour (en jours)
  * @param int nojour le numro d'ordre du jour reprsent
  * @param int debut le timestamp du dbut du sjour
  * @param int fin le timestamp de la fin du sjour
  * @param int proprio le propritaire du sjour
  * @param string title le numro de l'vnement
  * @param int urgent boolean en fait indiquant si le sjour est urgent ou non
  * @param int sortie_confirmee boolean  indiquant si la date de sortie du sjour a t confirme ou non
  * @param int sexe  boolean en fait indiquant le sexe du patient si aucun dossier n'a t attribu
  * @access public
  */
  function setOccupe($etabl, $ch, $lit, $doss, $duree, $nojour, $debut, $fin, $proprio, $title, $urgent, $sortie_confirmee, $sexe){
    if (!isset($this->occupation[$etabl][$ch][$lit])) {
      $index = 0;
    } else {
      $index = 1;
    }
    $this->occupation[$etabl][$ch][$lit][$index] = $doss;
    $this->duree[$etabl][$ch][$lit][$index] = $duree;
    $this->nojour[$etabl][$ch][$lit][$index] = $nojour;
    $this->debut[$etabl][$ch][$lit][$index] = $debut;
    $this->fin[$etabl][$ch][$lit][$index] = $fin;
    $this->proprio[$etabl][$ch][$lit][$index] = $proprio;
    $this->title[$etabl][$ch][$lit][$index] = $title;
    $this->urgent[$etabl][$ch][$lit][$index] = $urgent;
    $this->sortie_confirmee[$etabl][$ch][$lit][$index] = $sortie_confirmee;
    $this->sexe[$etabl][$ch][$lit][$index] = $sexe;
  }

  /**
  * Le lit en question est-il occup  la date du jour reprsent ?
  * @param int etabl le numro de l'tablissement
  * @param int ch le numro de la chambre
  * @param int lit le numro du lit
  * @access public
  * @return boolean
  */
  function isOccupe($etabl, $ch, $lit, $titre) {
    if (isset($this->occupation[$etabl][$ch][$lit])){
      if (in_array($titre, $this->title[$etabl][$ch][$lit]) && count($this->title[$etabl][$ch][$lit]) == 1){ 
        //Lit occup par le sjour en cours de modification
        return false;
      }
      else {  
        // si le lit est occup on renvoie l'occupation du sjour d'aprs
        if (isset($this->occupation[$etabl][$ch][$lit][1])){
          return $this->occupation[$etabl][$ch][$lit][1];
        }
        else {
          return $this->occupation[$etabl][$ch][$lit][0];
        }
      }
    }
    else {
      return false;
    }
  }

  /**
   * Le sjour est-il urgent ?
   * @param int etabl le numro de l'tablissement
   * @param int ch le numro de la chambre
   * @param int lit le numro du lit
   * @param int sejour l'index :0 (1 pour le sjour accol)
   */
  function isUrgent($etabl, $ch, $lit, $sejour = 0) {
    if (isset($this->urgent[$etabl][$ch][$lit][$sejour])) {
      return $this->urgent[$etabl][$ch][$lit][$sejour];
    } else {
      return 0;
    }
  }

  /**
   * La date de sortie du sjour est-elle confirme ?
   * @param int etabl le numro de l'tablissement
   * @param int ch le numro de la chambre
   * @param int lit le numro du lit
   * @param int sejour l'index :0 (1 pour le sjour accol)
   */
  function isSortieConfirmee($etabl, $ch, $lit, $sejour = 0) {
    if (isset($this->sortie_confirmee[$etabl][$ch][$lit][$sejour])) {
      return $this->sortie_confirmee[$etabl][$ch][$lit][$sejour];
    } else {
      return 0;
    }
  }

  /**
   * Le sjour bloqu est-il pour une femme ou un homme ?
   * N'est dfini que pour les sjours bloqus (sans attribution  un dossier particulier)
   * @param int etabl le numro de l'tablissement
   * @param int ch le numro de la chambre
   * @param int lit le numro du lit
   * @param int sejour l'index : 0 (1 pour le sjour accol)
   */
  function getSexe($etabl, $ch, $lit, $sejour = 0) {
    if (isset($this->sexe[$etabl][$ch][$lit][$sejour])) {
      return $this->sexe[$etabl][$ch][$lit][$sejour];
    } else {
      return "";
    }
  }

  /**
  * Renvoie le nom du mois  partir de son numro de 1 - 12
  * @param int mois le numro du mois
  * @access public
  * @static
  * @return string
  */
  function texteMois($mois){
    switch ($mois){
    case 1:
      return "janvier";
      break;
    case 2:
      return "fvrier";
      break;
    case 3:
      return "mars";
      break;
    case 4:
      return "avril";
      break;
    case 5:
      return "mai";
      break;
    case 6:
      return "juin";
      break;
    case 7:
      return "juillet";
      break;
    case 8:
      return "aot";
      break;
    case 9:
      return "septembre";
      break;
    case 10:
      return "octobre";
      break;
    case 11:
      return "novembre";
      break;
    case 12:
      return "dcembre";
      break;
    default:
      return false;
    }
  }
  /**
  * Renvoie l'abrviation du nom de mois  partir de son numro de 1 - 12
  * @param int mois le numro du mois
  * @access public
  * @static
  * @return string
  */
  function abrevMois($mois){
    switch ($mois){
    case 1:
      return "Jan";
      break;
    case 2:
      return "Fv";
      break;
    case 3:
      return "Mar";
      break;
    case 4:
      return "Avr";
      break;
    case 5:
      return "Mai";
      break;
    case 6:
      return "Juin";
      break;
    case 7:
      return "Juil";
      break;
    case 8:
      return "Aot";
      break;
    case 9:
      return "Sept";
      break;
    case 10:
      return "Oct";
      break;
    case 11:
      return "Nov";
      break;
    case 12:
      return "Dc";
      break;
    default:
      return false;
    }
  
  }
}


?>
