<?php
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
  
  // $Id: GrBoutonRadio.inc,v 1.2 2017/07/06 22:23:38 ob Exp $
/**
* Groupe de boutons radio
* @package outils
*/
/**
* Groupe de boutons radio
*
* Affiche un groupe de boutons radios rpondant au mme nom de champ et renvoyant des valeurs
* distinctes
* Doit tre inclus dans un contenant comme TPanel ou TPanelCadre
* <pre>
* Exemple :
* $c = new TPanel(1, 1);
* $x = new GrBoutonRadio("monbouton", $mavaleur, "", 1);
* $x->addRadio("Premire option", 1);
* $x->addRadio("Seconde option", 2);
* $c->addChamp($x);
* </pre>
* @author ob
* @package outils
*/
class GrBoutonRadio {
  /**
  * le nom du groupe de boutons
  * @var string
  */
  var $nom;
  /**
  * la valeur actuelle (celle qui dtermine le bouton activ)
  * @var mixed
  */
  var $valeur;
  /**
  * le contenu HTML du champ
  * @var string
  * @access private
  */
  var $contenu;
  /**
  * les instructions javascript  excuter sur changement
  * @var string
  */
  var $onchange;
  /**
  * faut-il afficher une "croix" de remise  zro du champ ?
  * si oui, le champ devra tre ajout  l'objet AutoCheckBox du formulaire.
  * @var boolean
  */
  var $zero;
  /**
  * faut-il un retour  la ligne aprs chaque option ?
  * @var boolean
  */
  var $br;

  /**
  * Indique s'il faut dsactiver ou non le champ
  * @var boolean
  */
  var $labellise;


  /**
  * @param string nom cf. variable de classe
  * @param mixed valeur cf. variable de classe
  * @param string onchange sf. variable de classe
  * @param boolean zero cf. variable de classe
  */
  function __construct($nom, $valeur, $onchange="", $zero=0, $labellise = false){
    $this->nom = $nom;
    $this->valeur = $valeur;
    $this->onchange = $onchange;
    $this->zero = $zero;
    $this->labellise = $labellise;
  }

  /**
  * Ajoute un bouton au groupe
  * @param string label le label du bouton
  * @param mixed mavaleur la valeur associe au bouton
  * @param boolean br faut-il un retour  la ligne aprs le bouton ?
  */
  function addRadio($label, $mavaleur, $br=false, $extra_texte = ""){
    $rd = new Radio($this->nom, $mavaleur, $label, ($this->valeur == $mavaleur), $br, $this->onchange, $this->labellise);
    $this->contenu .= $rd->getContenu();
    $this->contenu .= $extra_texte;    
    if ($br) {
      $this->br = true;
    }
  }

  /**
  * Renvoie le nom du champ
  * @return string
  */
  function getNom() {
    return $this->nom;
  }

  /**
  * Passe le contenu au contenant
  * En principe, est invoque par la mthode addChamp() du contenant
  * @access public
  * @return string le contenu du champ
  */
  function getContenu(){
    if ($this->zero && !$this->labellise) {
      $croix = "<A HREF=\"javascript:metAZero('$this->zero', '$this->nom')\">" .
               "<IMG BORDER=0 SRC=\"images/croixtr.gif\"></A>&nbsp;&nbsp;&nbsp;" .
               ($this->br ? "<BR>" : "") . "\n";
      $this->contenu = $croix . $this->contenu;
    }
    return $this->contenu;
  }
}
?>
