<?php
## Ce programme est un logiciel libre soumis aux termes de la
## licence GNU-GPL (http://www.gnu.org/copyleft/gpl.txt)
/**
* Table-formulaire reprsentant le corps d'un onglet
* @package outils
*/
/**
* Table-formulaire reprsentant le corps d'un onglet
*
* Contenant grant les onglets, instanci, le plus souvent  l'intrieur d'un cadronglet (contenant de 
* troisime niveau) une fois les onglets dessins et contenant 
* la totalit des objets graphiques constituant un formulaire.
* OngForm est un contenant de deuxime niveau, qui contient lui-mme des
* contenant de premier niveau du type "TPanel"
* <pre>
* Exemple :
* $of = new OngForm();
* $of->setTabParam($largeur, $couleurbande, 12);
* $of->setFormParam("donnees", $action, "post");
* $c = new TPanel(12, 1);
* $x = new Label ("Bonjour");
* $c->addChamp($x);
* $of->addChamp($x);
* </pre>
* @package outils
* @author ob
*/
class OngForm {
  /**
  * la largeur du contanant (en pixels ou en pourcents)
  * @var string
  */
  var $largeur;
  
  /**
  * la couleur du liser (bordure)
  * @var string
  */
  var $couleur;
  
  /**
  * le nom du formulaire
  * @var string
  */
  var $fName;
  
  /**
  * l'action du formulaire (telle qu'apparaissant dans le champ HTML <FORM action="...">)
  * @var string
  */
  var $fAction;
  
  /**
  * la mthode HTTP associe au formulaire (GET ou POST)
  * @var string
  */ 
  var $fMethod;
  
  /**
  * l'encodage des champs du formulaire (par exemple "multipart/form-data")
  * @var string
  */
  var $fenctype;
  
  /**
  * le nombre de colonnes du contenant
  * @var int
  */
  var $nbcol;
  
  /**
  * le corps du contenu
  * @var string
  * @access private
  */
  var $corps;
  
  /**
  * s'il y a lieu le champ qui doit recevoir le focus au moment de l'affichage du formulaire
  * @var string
  */
  var $focus;
  
  /**
  * largeur de l'cran (telle que stocke pour la session)
  * @var int
  */
  var $largeurecran;
  
  /**
  * id de l'objet html
  * @var int
  */
  var $fid;

  /**
  * Constructeur
  */
  function __construct(){

  }

  /**
  * Dfinition des paramtres de la table HTML
  * @param int largeur cf. variable de classe
  * @param int couleur cf. variable de classe
  * @param int nbcol cf. variable de classe
  * @param int largeurecran cf. variable de classe
  */
  function setTabParam($largeur, $couleur, $nbcol, $largeurecran = 800){
    $this->largeur = $largeur;
    $this->couleur = $couleur;
    $this->nbcol = $nbcol;
    if (! $largeurecran) $largeurecran = 800; // Au cas o 0 aurait t enregistr
    $this->largeurecran = $largeurecran;
  }
   
  /**
  * Dfinition des paramtres du formulaire
  * @param string name cf. variable fName
  * @param string action cf. variable fAction
  * @param string method cf. variable fMethod
  * @param string enctype cf. variable fenctype
  * @param string id cf. variable fid
  */
  function setFormParam($name = "", $action = "", $method = "", $enctype = "", $id = ""){
    $this->fName = $name;
    $this->fAction = $action;
    $this->fMethod = $method;
    $this->fenctype = $enctype;
    $this->fid = $id;
  }

  /**
  * Donne le focus au champ pass en paramtre
  * @param string champ le champ qui doit recevoir le focus
  */
  function setFocus($champ){
    $this->focus = $champ;
  }
  
  /**
  * Ajoute un champ au contenant, c'est--dire un contenant de premier niveau (p. ex TPanel)
  * @param object champ le champ ajout
  */
  function addChamp(&$champ){
    $this->corps .= $champ->TGetContenu();
  }

  /**
  * Ajout d'une page, ie un ensemble de champs, pour les formulaires complexes
  * @param object page la page ajoute
  */
  function addPage(&$page){
    $this->corps .= $page->TGetContenu();
  }

  /**
  * Ajout d'un retour  la ligne dans la table => les objets suivants s'affichent en-dessous
  * @param float fract la hauteur de la ligne est multiplie par ce nombre. Utile pour ajouter des 1/2 lignes si
  * l'on veut arer un peu la mise en page
  */
  function addLigne($fract=1){
    $this->corps .= "</TR>\n<TR><TD WIDTH=5><IMG BORDER=0 SRC=\"null.gif\" HEIGHT=" . (20 * $fract) . " WIDTH=5></TD>";
  }

  /**
  * Ajout d'une ligne vierge au formulaire (ne marche pas forcment... fonction pas rellement utilise)
  */
  function addLigneVierge(){
    $this->corps .= "</TR>\n<TR>";
  }
  
  /**
  * Renvoie le contenu, par exemple en le passant au cadronglet qui contient le tout
  * @return string
  */
  function getContenu(){
    //Adaptation de l'affichage au cas o $this->largeur est donn en pourcents
    if (preg_match("/%/", $this->largeur)){
      $malargeur = preg_replace("/%/", "", $this->largeur);
      $malargeur = $this->largeurecran * $malargeur / 100;
      $this->largeur = $malargeur;
    }
    $largcol = intval(($this->largeur - 7) / $this->nbcol);
    $entete = "<TABLE STYLE=\"border-color: $this->couleur; border-width:1px; border-style:solid;\" " .
              "CELLPADDING=0 CELLSPACING=0 BGCOLOR=\"white\" " . //Ce border pour les cases du form.
              "WIDTH=\"$this->largeur\">\n" .
              "<FORM NAME=\"$this->fName\" METHOD=\"$this->fMethod\" ACTION=\"$this->fAction\"" .
              ($this->fenctype ? " ENCTYPE=\"$this->fenctype\"" : "") .
              ($this->fid ? " ID=\"$this->fid\"" : "") .
              ">".
              "<TR><TD WIDTH=5 HEIGHT=1><IMG BORDER=0 SRC=\"null.gif\" HEIGHT=1></TD>\n";
    for ($i=0; $i < $this->nbcol; $i++){
      $entete .= "<TD HEIGHT=1 WIDTH=$largcol><IMG BORDER=0 SRC=\"null.gif\" HEIGHT=1></TD>\n";
    }
    $entete .= "</TR><TR><TD WIDTH=5><IMG BORDER=0 SRC=\"null.gif\" HEIGHT=20 WIDTH=5></TD>\n";

    $pied =   "</TR></FORM></TABLE>\n";
    if ($this->focus){
      $pied .= "<SCRIPT LANGUAGE=\"JavaScript\">" . 
               "for(i = 0; i < document.forms.$this->fName.elements.length; i++){" .
               "  if (document.forms.$this->fName.elements[i].name == '$this->focus'){" .
               "    document.forms.$this->fName.elements[i].focus();" .
               "    break;" .
               "  }" .
               "}" .
               "</SCRIPT>";
    }
    return $entete . $this->corps . $pied;
  }
  
  /**
  * Affiche directement le contenu (si on ne veut pas le passer  un autre objet
  */
  function imprime(){
    print $this->getContenu();
  }
}

?>
